/*----------------------------------------------------------------------------
 MINIGUI - Harbour Win32 GUI library source code

 Copyright 2002-2010 Roberto Lopez <harbourminigui@gmail.com>
 http://harbourminigui.googlepages.com/

 This program is free software; you can redistribute it and/or modify it under
 the terms of the GNU General Public License as published by the Free Software
 Foundation; either version 2 of the License, or (at your option) any later
 version.

 This program is distributed in the hope that it will be useful, but WITHOUT
 ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.

 You should have received a copy of the GNU General Public License along with
 this software; see the file COPYING. If not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA (or
 visit the web site http://www.gnu.org/).

 As a special exception, you have permission for additional uses of the text
 contained in this release of Harbour Minigui.

 The exception is that, if you link the Harbour Minigui library with other
 files to produce an executable, this does not by itself cause the resulting
 executable to be covered by the GNU General Public License.
 Your use of that executable is in no way restricted on account of linking the
 Harbour-Minigui library code into it.

 Parts of this project are based upon:

 "Harbour GUI framework for Win32"
  Copyright 2001 Alexander S.Kresin <alex@kresin.ru>
  Copyright 2001 Antonio Linares <alinares@fivetech.com>
 www - https://harbour.github.io/

 "Harbour Project"
 Copyright 1999-2023, https://harbour.github.io/

 "WHAT32"
 Copyright 2002 AJ Wos <andrwos@aust1.net>

 "HWGUI"
   Copyright 2001-2021 Alexander S.Kresin <alex@kresin.ru>

---------------------------------------------------------------------------*/

// Initialize g_hInstance on C-level
REQUEST GETINSTANCE

#include "SET_COMPILE_HMG_UNICODE.ch"

#include "minigui.ch"
#include "i_winuser.ch"

*------------------------------------------------------------------------------*
PROCEDURE Init
*------------------------------------------------------------------------------*
   LOCAL nCellForeColor := GetSysColor ( COLOR_HIGHLIGHTTEXT )
   LOCAL nCellBackColor := GetSysColor ( COLOR_HIGHLIGHT )

   STATIC _HMG_SysInit AS GLOBAL VALUE { Date(), Time() }

   PUBLIC _HMG_SYSDATA [ _HMG_SYSDATA_SIZE ]

// JP Drag Image
   _HMG_ActiveDragImageHandle := 0
// JP MDI
   _HMG_MainClientMDIHandle := 0
   _HMG_MainClientMDIName  := ""
   _HMG_ActiveMDIChildIndex := 0
   _HMG_BeginWindowMDIActive := .F.
   _HMG_MdiChildActive  := .F.
   _HMG_ActiveStatusHandle  := 0
// (JK) HMG 1.0 Experimental Build 6
   _HMG_ErrorLogFile := ""
   _HMG_CreateErrorlog := .T.

   _HMG_AutoScroll  := .T.
   _HMG_AutoAdjust  := .F.
   _HMG_AutoZooming := .F.

   _HMG_GlobalHotkeys := .F.
   _HMG_ProgrammaticChange := .T.
   _HMG_OwnerModalHandle := .F.
   _HMG_ListBoxDragNotification := 0

   _hmg_UserWindowHandle := 0
   _hmg_activemodalhandle := 0

   _HMG_InplaceParentHandle := 0
   _HMG_DefaultStatusBarMessage := 0
   _HMG_ActiveToolBarFormName := ''

   _HMG_IsXP := os_isWinXP()
   _HMG_IsXPorLater := IsWinXPorLater ()
   _HMG_IsThemed := IsThemed ()
   _HMG_IsBcc77 := iif( IsExe64(), ( "7.7" $ hb_Ccompiler() ), ( "7.7" $ hb_compiler() ) )

   _HMG_LANG_ID := ''

   _HMG_aLangButton    := {}
   _HMG_aLangLabel     := {}
   _HMG_aLangUser      := {}

   _HMG_aABMLangUser   := {}
   _HMG_aABMLangLabel  := {}
   _HMG_aABMLangButton := {}
   _HMG_aABMLangError  := {}

   _HMG_MESSAGE := Array ( 12 )
   _HMG_RPTDATA := Array ( 165 )

   _HMG_SetFocusExecuted := .F.

   _HMG_InteractiveCloseStarted := .F.
   _HMG_aEventInfo := {}

   _HMG_aUserBlocks := Array ( 7 )
   _HMG_lOOPEnabled := .F.

#ifdef _OBJECT_
   _HMG_bOnFormInit       := {|nIndex, cVar  | Do_OnWndInit   ( nIndex, cVar ) }
   _HMG_bOnFormDestroy    := {|nIndex        | Do_OnWndRelease( nIndex ) }
   _HMG_bOnControlInit    := {|nIndex, cVar  | Do_OnCtlInit   ( nIndex, cVar ) }
   _HMG_bOnControlDestroy := {|nIndex        | Do_OnCtlRelease( nIndex ) }
   _HMG_bOnWndLaunch      := {|hWnd, nMsg, wParam, lParam| Do_OnWndLaunch( hWnd, nMsg, wParam, lParam ) }
   _HMG_bOnCtlLaunch      := {|hWnd, nMsg, wParam, lParam| Do_OnCtlLaunch( hWnd, nMsg, wParam, lParam ) }
#endif
   _HMG_DateTextBoxActive := .F.

   _HMG_ThisFormName := Nil
   _HMG_ThisControlName := Nil

   _HMG_aBrowseSyncStatus := Array ( 2 )
   _HMG_BrowseSyncStatus := .F.
   _HMG_BrowseUpdateStatus := .F.

   _HMG_ActiveTabBold := Nil
   _HMG_ActiveTabItalic := Nil
   _HMG_ActiveTabUnderline := Nil
   _HMG_ActiveTabStrikeout := Nil
   _HMG_ActiveTabImages := Nil

   _HMG_ThisFormIndex := 0

   _HMG_InteractiveClose := 1

   _HMG_ThisEventType := ''

   _HMG_ExtendedNavigation := .F.

   _HMG_xContextMenuButtonIndex := 0

   _HMG_GridInplaceEdit := Array ( 3 )
   _HMG_GridInplaceEdit_StageEvent    := 0
   _HMG_GridInplaceEdit_ControlHandle := 0
   _HMG_GridInplaceEdit_GridIndex     := 0
   _HMG_IPE_COL := 1
   _HMG_IPE_ROW := 1

   _HMG_IPE_CANCELLED := .F.
   _HMG_GridNavigationMode := .F.

   _HMG_GridSelectedRowForeColor := { 0 , 0 , 0 }
   _HMG_GridSelectedRowBackColor := { 220 , 220 , 220 }
   _HMG_GridSelectedCellForeColor := nRGB2Arr ( nCellForeColor )
   _HMG_GridSelectedCellBackColor := nRGB2Arr ( nCellBackColor )

   _HMG_DialogCancelled := .F.
   _HMG_ActiveSplitBoxInverted := .F.

   _HMG_BRWLangButton	:= {}
   _HMG_BRWLangError	:= {}
   _HMG_BRWLangMessage	:= {}

   _HMG_ThisItemRowIndex := 0
   _HMG_ThisItemColIndex := 0
   _HMG_ThisItemCellRow  := 0
   _HMG_ThisItemCellCol  := 0
   _HMG_ThisItemCellWidth  := 0
   _HMG_ThisItemCellHeight  := 0
   _HMG_ThisItemCellValue  := Nil

   _HMG_ThisQueryData  := ""
   _HMG_ThisQueryRowIndex  := 0
   _HMG_ThisQueryColIndex  := 0

   _HMG_ThisType    := ''
   _HMG_ThisIndex    := 0
   _HMG_ShowContextMenus  := .T.
   _HMG_lMultiple   := .T.
   _HMG_IsMultiple   := Nil

   _HMG_DefaultIconName  := Nil
   _HMG_DefaultFontName  := _GetSysFont ()
   _HMG_DefaultFontSize  := Max( 9, GetDefaultFontSize () )

   _HMG_TempWindowName := ""

   _HMG_ActiveTabMultiline  := .F.

   _HMG_ActiveIniFile  := ""
   _HMG_ActiveHelpFile := ""
   _HMG_nTopic := 0
   _HMG_nMet   := 0

   _HMG_StatusItemCount := 0
   _HMG_ActiveMessageBarName := ""

   _HMG_ActiveSplitChildIndex := 0

   _HMG_xMenuType := ''
   _HMG_xMainMenuHandle := 0
   _HMG_xMainMenuParentHandle := 0
   _HMG_xMenuPopupLevel := 0
   _HMG_xMenuPopuphandle := Array ( 255 )
   _HMG_xMenuPopupCaption := Array ( 255 )
   _HMG_xMainMenuParentName := ""

   _HMG_xContextMenuHandle := 0
   _HMG_xContextMenuParentHandle := 0
   _HMG_xContextPopupLevel := 0
   _HMG_xContextPopuphandle := Array ( 255 )
   _HMG_xContextPopupCaption := Array ( 255 )
   _HMG_xContextMenuParentName := ""

   _HMG_aControlsContextMenu := {}
   _HMG_xControlsContextMenuID := 0

   _HMG_ActiveTreeValue := 0
   _HMG_ActiveTreeItemIds := .F.

   _HMG_ActiveTreeIndex := 0

   _HMG_ActiveTreeHandle := 0
   _HMG_NodeHandle := Array ( 255 )
   _HMG_NodeIndex := Nil
   _HMG_aTreeMap := {}
   _HMG_aTreeIdMap := {}

   _HMG_ActiveToolBarCaption := ''

   _HMG_SplitChildActive   := .F.

   _HMG_ToolBarActive      := .F.
   _HMG_ActiveToolBarExtend := .F.

   _HMG_ActiveFormNameBak  := ""

   _HMG_SplitLastControl   := ""

   _HMG_ActiveControlDef   := .F.
   _HMG_ActiveToolBarBreak := .F.
   _HMG_ActiveSplitBox     := .F.

   _HMG_ActiveSplitBoxParentFormName := ""

   _HMG_NotifyBalloonClick := ""

   _HMG_ActiveToolBarName := ""

   _HMG_MainWindowFirst := .T.
   _HMG_MainActive      := .F.
   _HMG_MainHandle      := 0
#ifdef _OBJECT_
   _HMG_MainCargo	:= oHmgData()
   _HMG_MainCargo:Version := hmg_Version()
#endif

   _HMG_MouseRow        := 0
   _HMG_MouseCol        := 0
   _HMG_MouseState      := 0

   _HMG_ActiveFormName  := ""
   _HMG_BeginWindowActive := .F.

   _HMG_ActiveFontName  := ""
   _HMG_ActiveFontSize  := 0

   _HMG_FrameLevel      := 0
   _HMG_ActiveFrameParentFormName := Array ( 128 )
   _HMG_ActiveFrameRow  := Array ( 128 )
   _HMG_ActiveFrameCol  := Array ( 128 )

   _HMG_ActiveDialogName   := ""
   _HMG_ActiveDialogHandle := 0
   _HMG_BeginDialogActive  := .F.
   _HMG_ModalDialogProcedure := ""
   _HMG_DialogProcedure    := ""
   _HMG_ModalDialogReturn  := 0
   _HMG_InitDialogProcedure := ""
   _HMG_ActiveTabnId    := 0
   _HMG_aDialogTemplate := {}
   _HMG_aDialogItems    := {}
   _HMG_DialogInMemory  := .F.
   _HMG_aDialogTreeItem := {}

   _HMG_ActiveDlgProcHandle  := 0
   _HMG_ActiveDlgProcMsg     := 0
   _HMG_ActiveDlgProcId      := 0
   _HMG_ActiveDlgProcNotify  := 0
   _HMG_ActiveDlgProcModal   := .F.

   _HMG_FldID := 0
   _HMG_aFolderInfo := {}

   _HMG_BeginPagerActive     := .F.
   _HMG_ActivePagerForm      := 0

#ifdef _TSBROWSE_
   _HMG_ActiveTBrowseName    := ""
   _HMG_ActiveTBrowseHandle  := 0
   _HMG_BeginTBrowseActive   := .F.
#endif

#ifdef _PROPGRID_
   _HMG_ActivePropGridHandle := 0
   _HMG_ActiveCategoryHandle := 0
   _HMG_ActivePropGridIndex  := 0
   _HMG_ActivePropGridArray  := {}
   _HMG_PGLangButton         := {}
   _HMG_PGLangError          := {}
   _HMG_PGLangMessage        := {}
#endif
   _HMG_BeginTabActive := .F.
   _HMG_ActiveTabPage  := 0
   _HMG_ActiveTabFullPageMap := {}
   _HMG_ActiveTabCaptions  := {}
   _HMG_ActiveTabCurrentPageMap := {}
   _HMG_ActiveTabName  := ""
   _HMG_ActiveTabParentFormName := ""
   _HMG_ActiveTabRow   := 0
   _HMG_ActiveTabCol   := 0
   _HMG_ActiveTabWidth := 0
   _HMG_ActiveTabHeight := 0
   _HMG_ActiveTabValue := 0
   _HMG_ActiveTabFontName := ""
   _HMG_ActiveTabFontSize := 0
   _HMG_ActiveTabToolTip  := ""
   _HMG_ActiveTabChangeProcedure := Nil
   _HMG_ActiveTabButtons  := .F.
   _HMG_ActiveTabFlat  := .F.
   _HMG_ActiveTabHotTrack := .F.
   _HMG_ActiveTabVertical := .F.
   _HMG_ActiveTabNoTabStop := .F.
   _HMG_ActiveTabMnemonic  := ""

   _HMG_IsModalActive := .F.

   _HMG_aScrollStep := { 0, 20 }
   _HMG_aFormDeleted  := {}
   _HMG_aFormNames    := {}
   _HMG_aFormHandles  := {}
   _HMG_aFormActive   := {}
   _HMG_aFormType     := {}
   _HMG_aFormParentHandle   := {}
   _HMG_aFormReleaseProcedure := {}
   _HMG_aFormInitProcedure  := {}
   _HMG_aFormToolTipHandle  := {}
   _HMG_aFormContextMenuHandle := {}
   _HMG_aFormMouseDragProcedure := {}
   _HMG_aFormSizeProcedure  := {}
   _HMG_aFormClickProcedure := {}
   _HMG_aFormMouseMoveProcedure := {}
   _HMG_aFormMoveProcedure  := {}
   _HMG_aFormDropProcedure  := {}
   _HMG_aFormBkColor  := {}
   _HMG_aFormPaintProcedure := {}
   _HMG_aFormNoShow  := {}
   _HMG_aFormNotifyIconName := {}
   _HMG_aFormNotifyIconToolTip := {}
   _HMG_aFormNotifyIconLeftClick := {}
   _HMG_aFormNotifyIconDblClick := {}
   _HMG_aFormGotFocusProcedure := {}
   _HMG_aFormLostFocusProcedure := {}
   _HMG_aFormReBarHandle  := {}
   _HMG_aFormNotifyMenuHandle := {}
   _HMG_aFormBrowseList  := {}
   _HMG_aFormSplitChildList := {}
   _HMG_aFormVirtualHeight  := {}
   _HMG_aFormVirtualWidth  := {}
   _HMG_aFormFocused  := {}
   _HMG_aFormScrollUp  := {}
   _HMG_aFormScrollDown  := {}
   _HMG_aFormScrollLeft  := {}
   _HMG_aFormScrollRight  := {}
   _HMG_aFormHScrollBox  := {}
   _HMG_aFormVScrollBox  := {}
   _HMG_aFormBrushHandle  := {}
   _HMG_aFormFocusedControl := {}
   _HMG_aFormGraphTasks  := {}
   _HMG_aFormMaximizeProcedure := {}
   _HMG_aFormMinimizeProcedure := {}
   _HMG_aFormRestoreProcedure := {}
   _HMG_aFormAutoRelease  := {}
   _HMG_aFormInteractiveCloseProcedure := {}
   _HMG_aFormMinMaxInfo  := {}
   _HMG_aFormActivateId  := {}
   _HMG_aFormMiscData1   := {}
   _HMG_aFormMiscData2   := {}

   _HMG_aControlDeleted  := {}
   _HMG_aControlType  := {}
   _HMG_aControlNames  := {}
   _HMG_aControlHandles  := {}
   _HMG_aControlParenthandles := {}
   _HMG_aControlIds  := {}
   _HMG_aControlProcedures  := {}
   _HMG_aControlPageMap  := {}
   _HMG_aControlValue  := {}
   _HMG_aControlInputMask  := {}
   _HMG_aControllostFocusProcedure := {}
   _HMG_aControlGotFocusProcedure := {}
   _HMG_aControlChangeProcedure := {}
   _HMG_aControlBkColor  := {}
   _HMG_aControlFontColor  := {}
   _HMG_aControlDblClick  := {}
   _HMG_aControlHeadClick  := {}
   _HMG_aControlRow  := {}
   _HMG_aControlCol  := {}
   _HMG_aControlWidth  := {}
   _HMG_aControlHeight  := {}
   _HMG_aControlSpacing  := {}
   _HMG_aControlContainerRow := {}
   _HMG_aControlContainerCol := {}
   _HMG_aControlPicture  := {}
   _HMG_aControlContainerHandle := {}
   _HMG_aControlFontName  := {}
   _HMG_aControlFontSize  := {}
   _HMG_aControlToolTip  := {}
   _HMG_aControlRangeMin  := {}
   _HMG_aControlRangeMax  := {}
   _HMG_aControlCaption  := {}
   _HMG_aControlVisible  := {}
   _HMG_aControlHelpId  := {}
   _HMG_aControlFontHandle  := {}
   _HMG_aControlFontAttributes := {}
   _HMG_aControlBrushHandle := {}
   _HMG_aControlEnabled  := {}
   _HMG_aControlMiscData1 := {}
   _HMG_aControlMiscData2 := {}

   _HMG_ListBoxDragNotification := _GetDDLMessage()
   _HMG_FindReplaceOptions := Array ( 6 )
   _HMG_CharRange_Min := 0
   _HMG_CharRange_Max := 0
   _HMG_MsgIDFindDlg := RegisterFindMsgString()

#ifdef _USERINIT_
   _HMG_aCustomEventProcedure := {}
   _HMG_aCustomPropertyProcedure := {}
   _HMG_aCustomMethodProcedure := {}
   _HMG_UserComponentProcess := .F.
#endif

   _HMG_ParentWindowActive  := .F.
   _HMG_aErrorBlocks := Array ( 3 )
   _HMG_lOnErrorStop := .F.
   _HMG_ProceedEachRadioButtonEvent := .T.

#ifdef _PANEL_
   _HMG_LoadWindowRow  := -1
   _HMG_LoadWindowCol  := -1
   _HMG_LoadWindowWidth  := -1
   _HMG_LoadWindowHeight  := -1
#endif
#ifdef _HMG_COMPAT_
   _HMG_StopWindowEventProcedure := {}
   _HMG_StopControlEventProcedure := {}
   _HMG_LastActiveFormIndex := 0
   _HMG_LastActiveControlIndex := 0
#endif

#if ! defined( __XHARBOUR__ ) && ( ( __HARBOUR__ - 0 ) > 0x030100 )

#ifdef UNICODE
   Set ( _SET_CODEPAGE, "UTF8" )
#else
   InitCodePage()
#endif

#endif

   InitMessages()

   ResetGlobalListener() // set default Events function

   _HMG_IsMultiple := IsExeRunning ( StrTran( GetExeFileName (), '\', '_' ) )

   _SetErrorLogFile( _GetErrorLogFile() ) // set default ErrorLog file

   Set( _SET_DELIMITERS, .T. ) // set standard delimiters

RETURN

*------------------------------------------------------------------------------*
FUNCTION TimeFromStart
*------------------------------------------------------------------------------*
   LOCAL aData
   LOCAL aStart := _SetGetGlobal( "_HMG_SysInit" )
   LOCAL cText := ""
   LOCAL n

   aData := _hmg_Elapsed( aStart [1], Date(), aStart [2], Time() )
   FOR n := 1 TO 4
      cText += hb_ntos( aData[ n ] ) + " "
      cText += iif( n == 1, "days ", iif( n == 2, "hours ", iif( n == 3, "mins ", "secs" ) ) )
   NEXT

RETURN cText

*------------------------------------------------------------------------------*
STATIC FUNCTION _hmg_Elapsed( dStart, dEnd, cTimeStart, cTimeEnd )
*------------------------------------------------------------------------------*
   LOCAL aRetVal [4]
   LOCAL nTotalSec, nCtr, nConstant
   LOCAL nTemp

   nTotalSec := ( dEnd - dStart ) * 86400 + ;
      Val( cTimeEnd ) *  3600 + ;
      Val( SubStr( cTimeEnd, At( ":", cTimeEnd ) + 1, 2 ) ) * 60 + ;
      iif( RAt( ":", cTimeEnd ) == At( ":", cTimeEnd ), 0, ;
      Val( SubStr( cTimeEnd, RAt( ":", cTimeEnd ) + 1 ) ) ) - ;
      Val( cTimeStart ) * 3600 - ;
      Val( SubStr( cTimeStart, At( ":", cTimeStart ) + 1, 2 ) ) * 60 - ;
      iif( RAt( ":", cTimeStart ) == At( ":", cTimeStart ), 0, ;
      Val( SubStr( cTimeStart, RAt( ":", cTimeStart ) + 1 ) ) )

   nTemp := nTotalSec

   FOR nCtr := 1 TO 4
      nConstant := iif( nCtr == 1, 86400, iif( nCtr == 2, 3600, iif( nCtr == 3, 60, 1 ) ) )
      aRetVal[ nCtr ] := Int( nTemp / nConstant )
      nTemp -= aRetVal[ nCtr ] * nConstant
   NEXT

RETURN aRetVal


#ifndef UNICODE
#include "fileio.ch"

#define _UTF8_BOM       e"\xEF\xBB\xBF"  /* hb_utf8Chr( 0xFEFF ) */
#ifdef __XHARBOUR__
#xtranslate hb_eol() => hb_OsNewLine()
#ifdef __XCC__
#xtranslate hb_BLen( <c> ) => Len( <c> )
#endif
#endif
*------------------------------------------------------------------------------*
FUNCTION HMG_CreateFile_UTF16LE_BOM( cFile )
*------------------------------------------------------------------------------*
   LOCAL hFile
   LOCAL cPOT := _UTF8_BOM + hb_eol()

   IF ( hFile := FOpen( cFile, FO_CREAT + FO_TRUNC + FO_WRITE + FO_EXCLUSIVE ) ) != NIL
      IF FWrite( hFile, cPOT ) != hb_BLen( cPOT )
         FClose( hFile )
         hFile := NIL
      ENDIF
   ENDIF

RETURN hFile

#else

/*
 * Registry Access Functions
*/

FUNCTION IsRegistryKey( nKey, cRegKey )

   LOCAL pKeyHandle
   LOCAL lExist

   lExist := win_regOpenKeyEx( nKey, cRegKey, 0, KEY_SET_VALUE, @pKeyHandle )

   win_regCloseKey( pKeyHandle )

RETURN lExist


FUNCTION CreateRegistryKey( nKey, cRegKey )

   LOCAL pKeyHandle
   LOCAL lSuccess

   lSuccess := win_regCreateKeyEx( nKey, cRegKey, 0, 0, 0, KEY_SET_VALUE, 0, @pKeyHandle )

   win_regCloseKey( pKeyHandle )

RETURN lSuccess


#define KEY_WOW64_64KEY 0x0100

FUNCTION GetRegistryValue( nKey, cRegKey, cRegVar, cType, nRegSam )

   LOCAL uVal
   LOCAL xKey

   DEFAULT cRegVar TO '', cType TO 'C'

   IF HB_ISNIL( nRegSam ) .AND. IsWin64()
      nRegSam := KEY_WOW64_64KEY
   ENDIF

   xKey := win_regGet( nKey, cRegKey, cRegVar, , nRegSam )

   DO CASE
      CASE cType == 'N'
         uVal := 0
      CASE cType == 'D'
         uVal := BLANK_DATE
      CASE cType == 'L'
         uVal := .F.
      OTHERWISE
         uVal := ''
   ENDCASE

   IF xKey != NIL .AND. cType == ValType( xKey )

      uVal := win_regGet( nKey, cRegKey, cRegVar, uVal, nRegSam )

   ENDIF

RETURN uVal


FUNCTION SetRegistryValue( nKey, cRegKey, cRegVar, uVal, nRegSam )

   DEFAULT cRegVar TO ''

   IF HB_ISNIL( nRegSam ) .AND. IsWin64()
      nRegSam := KEY_WOW64_64KEY
   ENDIF

RETURN( win_regSet( nKey, cRegKey, cRegVar, uVal, , nRegSam ) )


FUNCTION DeleteRegistryVar( nKey, cRegKey, cRegVar, nRegSam )

   LOCAL pKeyHandle
   LOCAL lSuccess := .F.

   DEFAULT cRegVar TO '', nRegSam TO 0

   IF Empty( nRegSam ) .AND. IsWin64()
      nRegSam := KEY_WOW64_64KEY
   ENDIF

   nKey := iif( nKey == NIL, HKEY_CURRENT_USER, nKey )

   IF win_regOpenKeyEx( nKey, cRegKey, 0, hb_bitOr( KEY_SET_VALUE, nRegSam ), @pKeyHandle )
      lSuccess := win_regDeleteValue( pKeyHandle, cRegVar )
      win_regCloseKey( pKeyHandle )
   ENDIF

RETURN lSuccess


FUNCTION DeleteRegistryKey( nKey, cRegKey )

RETURN( win_regDeleteKey( nKey, cRegKey ) )

#endif

#if ! defined( __XHARBOUR__ ) && ( ( __HARBOUR__ - 0 ) > 0x030100 )
#ifndef UNICODE
*------------------------------------------------------------------------------*
STATIC PROCEDURE InitCodePage
*------------------------------------------------------------------------------*
   LOCAL cLang

   IF Empty( cLang := hb_UserLang() )

      SET CODEPAGE TO ENGLISH

   ELSE

      DO CASE

      CASE "es" $ cLang
         SET CODEPAGE TO SPANISH

      CASE "pt" $ cLang
         SET CODEPAGE TO PORTUGUESE

      CASE "de" $ cLang
         SET CODEPAGE TO GERMAN

      CASE "el" $ cLang
         SET CODEPAGE TO GREEK

      CASE "ru" $ cLang
         SET CODEPAGE TO RUSSIAN

      CASE "uk" $ cLang
         SET CODEPAGE TO UKRAINIAN

      CASE "pl" $ cLang
         SET CODEPAGE TO POLISH

      CASE "sl" $ cLang
         SET CODEPAGE TO SLOVENIAN

      CASE "sr" $ cLang
         SET CODEPAGE TO SERBIAN

      CASE "bg" $ cLang
         SET CODEPAGE TO BULGARIAN

      CASE "hu" $ cLang
         SET CODEPAGE TO HUNGARIAN

      CASE "cs" $ cLang
         SET CODEPAGE TO CZECH

      CASE "sk" $ cLang
         SET CODEPAGE TO SLOVAK

      CASE "nl" $ cLang
         SET CODEPAGE TO DUTCH

      CASE "fi" $ cLang
         SET CODEPAGE TO FINNISH

      CASE "sv" $ cLang
         SET CODEPAGE TO SWEDISH

      ENDCASE

   ENDIF

RETURN

#endif
#endif
*------------------------------------------------------------------------------*
FUNCTION _GetSysFont()
*------------------------------------------------------------------------------*

   IF _HMG_IsXPorLater
      RETURN GetDefaultFontName()
   ENDIF

RETURN "MS Sans Serif"  // Win NT, 9x

*------------------------------------------------------------------------------*
#ifdef _MULTILINGUAL_
PROCEDURE InitMessages( cLang )
#else
PROCEDURE InitMessages
#endif
*------------------------------------------------------------------------------*

   // MISC MESSAGES (ENGLISH DEFAULT)

   _HMG_MESSAGE [1] := 'Are you sure ?'
   _HMG_MESSAGE [2] := 'Close Window'
   _HMG_MESSAGE [3] := 'Close not allowed'
   _HMG_MESSAGE [4] := 'Program Already Running'
   _HMG_MESSAGE [5] := 'Edit'
   _HMG_MESSAGE [6] := 'Ok'
   _HMG_MESSAGE [7] := 'Cancel'
   _HMG_MESSAGE [8] := 'Apply'
   _HMG_MESSAGE [9] := 'Pag.'
   _HMG_MESSAGE [10] := 'Attention'
   _HMG_MESSAGE [11] := 'Information'
   _HMG_MESSAGE [12] := 'Stop'

   // BROWSE MESSAGES (ENGLISH DEFAULT)

   _HMG_BRWLangButton := { ;
      "Append"  , ;
      "Edit"    , ;
      "&Cancel" , ;
      "&OK" }
   _HMG_BRWLangError  := { ;
      "Window: "                                              , ;
      " is not defined. Program terminated"                   , ;
      "MiniGUI Error"                                         , ;
      "Control: "                                             , ;
      " Of "                                                  , ;
      " Already defined. Program terminated"                  , ;
      "Browse: Type Not Allowed. Program terminated"          , ;
      "Browse: Append Clause Can't Be Used With Fields Not Belonging To Browse WorkArea.", ;
      "Record Is Being Edited By Another User"                , ;
      "Warning"                                               , ;
      "Invalid Entry" }
   _HMG_BRWLangMessage := { 'Are you sure ?' , 'Delete Record' }

   // EDIT MESSAGES (ENGLISH DEFAULT)

   _HMG_aABMLangUser   := { ;
      Chr( 13 ) + "Delete record" + Chr( 13 ) + "Are you sure ?" + Chr( 13 )            , ;
      Chr( 13 ) + "Index file missing" + Chr( 13 ) + "Can`t do search" + Chr( 13 )      , ;
      Chr( 13 ) + "Can`t find index field" + Chr( 13 ) + "Can`t do search" + Chr( 13 )  , ;
      Chr( 13 ) + "Can't do search by" + Chr( 13 ) + "fields memo or logic" + Chr( 13 ) , ;
      Chr( 13 ) + "Record not found" + Chr( 13 )                                        , ;
      Chr( 13 ) + "To many cols" + Chr( 13 ) + "The report can't fit in the sheet" + Chr( 13 ) }

   _HMG_aABMLangLabel := {    ;
      "Record"              , ;
      "Record count"        , ;
      "       (New)"        , ;
      "      (Edit)"        , ;
      "Enter record number" , ;
      "Find"                , ;
      "Search text"         , ;
      "Search date"         , ;
      "Search number"       , ;
      "Report definition"   , ;
      "Report columns"      , ;
      "Available columns"   , ;
      "Initial record"      , ;
      "Final record"        , ;
      "Report of "          , ;
      "Date:"               , ;
      "Initial record:"     , ;
      "Final record:"       , ;
      "Ordered by:"         , ;
      "Yes"                 , ;
      "No"                  , ;
      "Page "               , ;
      " of " }

   _HMG_aABMLangButton := { ;
      "Close"    , ;
      "New"      , ;
      "Edit"     , ;
      "Delete"   , ;
      "Find"     , ;
      "Goto"     , ;
      "Report"   , ;
      "First"    , ;
      "Previous" , ;
      "Next"     , ;
      "Last"     , ;
      "Save"     , ;
      "Cancel"   , ;
      "Add"      , ;
      "Remove"   , ;
      "Print"    , ;
      "Close" }
   _HMG_aABMLangError := { ;
      "EDIT, workarea name missing"                              , ;
      "EDIT, this workarea has more than 16 fields"              , ;
      "EDIT, refresh mode out of range (please report bug)"      , ;
      "EDIT, main event number out of range (please report bug)" , ;
      "EDIT, list event number out of range (please report bug)" }

   // EDIT EXTENDED (ENGLISH DEFAULT)

   _HMG_aLangButton := {    ;
      "&Close",             ; // 1
      "&New",               ; // 2
      "&Modify",            ; // 3
      "&Delete",            ; // 4
      "&Find",              ; // 5
      "&Print",             ; // 6
      "&Cancel",            ; // 7
      "&Ok",                ; // 8
      "&Copy",              ; // 9
      "&Activate Filter",   ; // 10
      "&Deactivate Filter", ; // 11
      "&Restore",           ; // 12
		"&Retry"            } // 13

   _HMG_aLangLabel := { ;
      "None",                       ; // 1
      "Record",                     ; // 2
      "Total",                      ; // 3
      "Active order",               ; // 4
      "Options",                    ; // 5
      "New record",                 ; // 6
      "Modify record",              ; // 7
      "Select record",              ; // 8
      "Find record",                ; // 9
      "Print options",              ; // 10
      "Available fields",           ; // 11
      "Fields to print",            ; // 12
      "Available printers",         ; // 13
      "First record to print",      ; // 14
      "Last record to print",       ; // 15
      "Delete record",              ; // 16
      "Preview",                    ; // 17
      "View page thumbnails",       ; // 18
      "Filter Condition: ",         ; // 19
      "Filtered: ",                 ; // 20
      "Filtering Options" ,         ; // 21
      "Database Fields" ,           ; // 22
      "Comparison Operator",        ; // 23
      "Filter Value",               ; // 24
      "Select Field To Filter",     ; // 25
      "Select Comparison Operator", ; // 26
      "Equal",                      ; // 27
      "Not Equal",                  ; // 28
      "Greater Than",               ; // 29
      "Lower Than",                 ; // 30
      "Greater or Equal Than",      ; // 31
      "Lower or Equal Than"         } // 32
   _HMG_aLangUser := { ;
      CRLF + "Can't find an active area.   "  + CRLF + "Please select any area before call EDIT   " + CRLF,       ; // 1
      "Type the field value (any text)",                                                                          ; // 2
      "Type the field value (any number)",                                                                        ; // 3
      "Select the date",                                                                                          ; // 4
      "Check for true value",                                                                                     ; // 5
      "Enter the field value",                                                                                    ; // 6
      "Select any record and press OK",                                                                           ; // 7
      CRLF + "You are going to delete the active record   " + CRLF + "Are you sure?    " + CRLF,                  ; // 8
      CRLF + "There isn't any active order   " + CRLF + "Please select one   " + CRLF,                            ; // 9
      CRLF + "Can't do searches by fields memo or logic   " + CRLF,                                               ; // 10
      CRLF + "Record not found   " + CRLF,                                                                        ; // 11
      "Select the field to include to list",                                                                      ; // 12
      "Select the field to exclude from list",                                                                    ; // 13
      "Select the printer",                                                                                       ; // 14
      "Push button to include field",                                                                             ; // 15
      "Push button to exclude field",                                                                             ; // 16
      "Push button to select the first record to print",                                                          ; // 17
      "Push button to select the last record to print",                                                           ; // 18
      CRLF + "No more fields to include   " + CRLF,                                                               ; // 19
      CRLF + "First select the field to include   " + CRLF,                                                       ; // 20
      CRLF + "No more fields to exlude   " + CRLF,                                                                ; // 21
      CRLF + "First select th field to exclude   " + CRLF,                                                        ; // 22
      CRLF + "You don't select any field   " + CRLF + "Please select the fields to include on print   " + CRLF,   ; // 23
      CRLF + "Too many fields   " + CRLF + "Reduce number of fields   " + CRLF,                                   ; // 24
      CRLF + "Printer not ready   " + CRLF,                                                                       ; // 25
      "Ordered by",                                                                                               ; // 26
      "From record",                                                                                              ; // 27
      "To record",                                                                                                ; // 28
      "Yes",                                                                                                      ; // 29
      "No",                                                                                                       ; // 30
      "Page:",                                                                                                    ; // 31
      CRLF + "Please select a printer   " + CRLF,                                                                 ; // 32
      "Filtered by",                                                                                              ; // 33
      CRLF + "There is an active filter    " + CRLF,                                                              ; // 34
      CRLF + "Can't filter by memo fields    " + CRLF,                                                            ; // 35
      CRLF + "Select the field to filter    " + CRLF,                                                             ; // 36
      CRLF + "Select any operator to filter    " + CRLF,                                                          ; // 37
      CRLF + "Type any value to filter    " + CRLF,                                                               ; // 38
      CRLF + "There isn't any active filter    " + CRLF,                                                          ; // 39
      CRLF + "Deactivate filter?   " + CRLF,                                                                      ; // 40
      CRLF + "Record locked by another user    " + CRLF,                                                          ; // 41
      CRLF + "You are going to restore the deleted record   " + CRLF + "Are you sure?    " + CRLF                 } // 42

#ifdef _MULTILINGUAL_

   IF HB_ISNIL ( cLang ) .OR. ValType ( cLang ) != "C"
      IF _HMG_LANG_ID == 'FI'  // FINNISH - Language Is Not Supported By hb_langSelect() Function
         cLang := 'FI'
      ELSE
         cLang := Upper( Left( Set ( _SET_LANGUAGE ), 2 ) )
      ENDIF
   ENDIF

   DO CASE

   CASE cLang == "CS"  // Czech
      /////////////////////////////////////////////////////////////
      // CZECH
      ////////////////////////////////////////////////////////////

      // MISC MESSAGES

      _HMG_MESSAGE [1] := 'Jste si jist(a)?'
      _HMG_MESSAGE [2] := 'Zavi okno'
      _HMG_MESSAGE [3] := 'Uzaven zakzno'
      _HMG_MESSAGE [4] := 'Program u b'
      _HMG_MESSAGE [5] := 'prava'
      _HMG_MESSAGE [6] := 'Ok'
      _HMG_MESSAGE [7] := 'Storno'
      _HMG_MESSAGE [8] := 'Apply'
      _HMG_MESSAGE [9] := 'Str.'
      _HMG_MESSAGE [10] := 'Attention'
      _HMG_MESSAGE [11] := 'Information'
      _HMG_MESSAGE [12] := 'Stop'

      // BROWSE MESSAGES

      _HMG_BRWLangButton := { "Append"  , ;
         "Edit"    , ;
         "&Cancel"  , ;
         "&OK"       }
      _HMG_BRWLangError  := { "Okno: "                                              , ;
         " nen definovno. Program ukonen"                   , ;
         "MiniGUI Error"                                         , ;
         "Prvek: "                                             , ;
         " z "                                                  , ;
         " u definovn. Program ukonen"                  , ;
         "Browse: Typ nepovolen. Program ukonen"          , ;
         "Browse: Append frzi nelze pout s poli nepatcmi do Browse pracovn oblasti. Program ukonen", ;
         "Zznam edituje jin uivatel"                , ;
         "Varovn"                                              , ;
         "Chybn vstup"                                          }
      _HMG_BRWLangMessage := { 'Jste si jist(a)?' , 'Smazat zznam' }

      // EDIT MESSAGES

      _HMG_aABMLangUser   := { Chr( 13 ) + "Smazat zznam" + Chr( 13 ) + "Jste si jist(a)?" + Chr( 13 ) , ;
         Chr( 13 ) + "Chyb indexov soubor" + Chr( 13 ) + "Nemohu hledat" + Chr( 13 )            , ;
         Chr( 13 ) + "Nemohu najt indexovan pole" + Chr( 13 ) + "Nemohu hledat" + Chr( 13 )        , ;
         Chr( 13 ) + "Nemohu hledat podle" + Chr( 13 ) + "pole memo nebo logick" + Chr( 13 )       , ;
         Chr( 13 ) + "Zznam nenalezen" + Chr( 13 )                                        , ;
         Chr( 13 ) + "Pli mnoho sloupc" + Chr( 13 ) + "Sestava se nevejde na plochu" + Chr( 13 ) }

      _HMG_aABMLangLabel  := { "Zznam"      , ;
         "Poet zznam"         , ;
         "      (Nov)"          , ;
         "     (prava)"         , ;
         "Zadejte slo zznamu" , ;
         "Hledej"                , ;
         "Hledan text"          , ;
         "Hledan datum"         , ;
         "Hledan slo"         , ;
         "Definice sestavy"      , ;
         "Sloupce sestavy"       , ;
         "Dostupn sloupce"      , ;
         "Prvn zznam"          , ;
         "Posledn zznam"       , ;
         "Sestava "              , ;
         "Datum:"                , ;
         "Prvn zznam:"         , ;
         "Posledn zznam:"      , ;
         "Tdno dle:"          , ;
         "Ano"                   , ;
         "Ne"                    , ;
         "Strana "               , ;
         " z "                   }

      _HMG_aABMLangButton := { "Zavt"    , ;
         "Nov"      , ;
         "prava"    , ;
         "Sma"      , ;
         "Najdi"     , ;
         "Jdi"       , ;
         "Sestava"   , ;
         "Prvn"     , ;
         "Pedchoz" , ;
         "Dal"     , ;
         "Posledn"  , ;
         "Ulo"      , ;
         "Storno"    , ;
         "Pidej"    , ;
         "Odstra"   , ;
         "Tisk"      , ;
         "Zavi"     }
      _HMG_aABMLangError  := { "EDIT, chyb jmno pracovn oblasti" , ;
         "EDIT, pracovn oblast m vc jak 16 pol"              , ;
         "EDIT, refresh mode mimo rozsah (prosm, nahlaste chybu)"      , ;
         "EDIT, hlavn event slo mimo rozsah (prosm, nahlaste chybu)" , ;
         "EDIT, list event slomimo rozsah (prosm, nahlaste chybu)"  }

      // EDIT EXTENDED

      _HMG_aLangButton := { ;
      "&Zavi",            ; // 1
      "&Nov",             ; // 2
      "&prava",           ; // 3
      "S&ma  ",            ; // 4
      "Na&jdi",            ; // 5
      "&Tisk",             ; // 6
      "&Storno",           ; // 7
      "&Ok",               ; // 8
      "&Kopruj",          ; // 9
      "Aktivuj &filtr",    ; // 10
      "&Vypni filtr",      ; // 11
      "&Restore",          ; // 12
		"Retry"              } // 13

      _HMG_aLangLabel := {            ;
         "dn",                        ; // 1
      "Zznam",                       ; // 2
      "Suma",                         ; // 3
      "Aktivn tdn",              ; // 4
      "Volby",                        ; // 5
      "Nov zznam",                  ; // 6
      "Uprav zznam",                 ; // 7
      "Vyber zznam",                 ; // 8
      "Najdi zznam",                 ; // 9
      "Tiskni volby",                 ; // 10
      "Dostupn pole",                ; // 11
      "Pole k tisku",                 ; // 12
      "Dostupn tiskrny",            ; // 13
      "Prvn zznam k tisku",         ; // 14
      "Posledn zznam k tisku",      ; // 15
      "Sma zznam",                  ; // 16
      "Nhled",                       ; // 17
      "Zobraz miniatury stran",       ; // 18
      "Filtr: ",                      ; // 19
      "Filtrovn: ",                  ; // 20
      "Volby filtru",                 ; // 21
      "Pole databze",                ; // 22
      "Opertor porovnn",           ; // 23
      "Hodnota filtru",               ; // 24
      "Vyber pole do filtru",         ; // 25
      "Vyber opertor porovnn",     ; // 26
      "rovno",                        ; // 27
      "nerovno",                      ; // 28
      "vt ne",                    ; // 29
      "men ne",                    ; // 30
      "vt nebo rovno ne",         ; // 31
      "men nebo rovno ne"          } // 32
      _HMG_aLangUser := { ;
         CRLF + "Nelze najt aktivn oblast   "  + CRLF + "Prosm vyberte nkterou ped volnm EDIT   " + CRLF,         ; // 1
      "Zadejte hodnotu pole (libovoln text)",                                                                        ; // 2
      "Zadejte hodnotu pole (libovoln slo)",                                                                       ; // 3
      "Vyberte datum",                                                                                                ; // 4
      "Zatrhnte pro hodnotu true",                                                                                   ; // 5
      "Zadejte hodnotu pole",                                                                                         ; // 6
      "Vyberte jakkoliv zznam s stisknte OK",                                                                      ; // 7
      CRLF + "Chcete smazat tento zznam  " + CRLF + "Jste si jist(a)?    " + CRLF,                                   ; // 8
      CRLF + "Nen vybrno dn tdn   " + CRLF + "Prosm zvolte jedno   " + CRLF,                                ; // 9
      CRLF + "Nelze hledat podle pole memo nebo logic   " + CRLF,                                                     ; // 10
      CRLF + "Zznam nenalezen   " + CRLF,                                                                            ; // 11
      "Vyberte pole k zaazen do seznamu",                                                                           ; // 12
      "Vyberte pole k vyazen ze seznamu",                                                                           ; // 13
      "Vyberte tiskrnu",                                                                                             ; // 14
      "Stisknte tlatko pro zaazen pole",                                                                         ; // 15
      "Stisknt tlatko k vyazen pole",                                                                           ; // 16
      "Stisknte tlatko k vbru prvnho zznamu k tisku",                                                          ; // 17
      "Stisknt tlatko k vbru poslednho zznamu k tisku",                                                       ; // 18
      CRLF + "K zaazen nezbvaj pole   " + CRLF,                                                                   ; // 19
      CRLF + "Prvn vbr pole k zaazen   " + CRLF,                                                                 ; // 20
      CRLF + "Nelze vyadit dal pole   " + CRLF,                                                                    ; // 21
      CRLF + "Prvn vbr pole k vyazen   " + CRLF,                                                                 ; // 22
      CRLF + "Nebylo vybrno dn pole   " + CRLF + "Prosm vyberte pole pro zaazen do tisku   " + CRLF,           ; // 23
      CRLF + "Pli mnoho pol   " + CRLF + "odeberte nkter pole   " + CRLF,                                       ; // 24
      CRLF + "Tiskrna nen pipravena   " + CRLF,                                                                    ; // 25
      "Tdno dle",                                                                                                  ; // 26
      "Od zznamu",                                                                                                   ; // 27
      "Do zznamu",                                                                                                   ; // 28
      "Ano",                                                                                                          ; // 29
      "Ne",                                                                                                           ; // 30
      "Strana:",                                                                                                      ; // 31
      CRLF + "Prosm vyberte tiskrnu   " + CRLF,                                                                     ; // 32
      "Filtrovno dle",                                                                                               ; // 33
      CRLF + "Filtr nen aktivn    " + CRLF,                                                                         ; // 34
      CRLF + "Nelze filtrovat podle memo    " + CRLF,                                                                 ; // 35
      CRLF + "Vyberte pole do filtru    " + CRLF,                                                                     ; // 36
      CRLF + "Vybarte opertor do filtru    " + CRLF,                                                                 ; // 37
      CRLF + "Zadejte hodnotu do filtru    " + CRLF,                                                                  ; // 38
      CRLF + "Nen dn aktivn filtr    " + CRLF,                                                                   ; // 39
      CRLF + "Deactivovat filtr?   " + CRLF,                                                                          ; // 40
      CRLF + "Zznam uzamen jinm uivatelem  " + CRLF,                                                              ; // 41
      CRLF + "You are going to restore the deleted record   " + CRLF + "Are you sure?    " + CRLF                     } // 42

   CASE cLang == "HR"  // Croatian
      /////////////////////////////////////////////////////////////
      // CROATIAN
      ////////////////////////////////////////////////////////////

      // MISC MESSAGES

      _HMG_MESSAGE [1] := 'Are you sure ?'
      _HMG_MESSAGE [2] := 'Zatvori prozor'
      _HMG_MESSAGE [3] := 'Zatvaranje nije dozvoljeno'
      _HMG_MESSAGE [4] := 'Program je ve pokrenut'
      _HMG_MESSAGE [5] := 'Uredi'
      _HMG_MESSAGE [6] := 'U redu'
      _HMG_MESSAGE [7] := 'Prekid'
      _HMG_MESSAGE [8] := 'Apply'
      _HMG_MESSAGE [9] := 'Pag.'
      _HMG_MESSAGE [10] := 'Attention'
      _HMG_MESSAGE [11] := 'Information'
      _HMG_MESSAGE [12] := 'Stop'

      // BROWSE MESSAGES

      _HMG_BRWLangButton := { "Append"  , ;
         "Edit"    , ;
         "&Cancel"  , ;
         "&OK"       }
      _HMG_BRWLangError  := { "Window: "                                           , ;
         " is not defined. Program terminated"                   , ;
         "MiniGUI Error"                                         , ;
         "Control: "                                             , ;
         " Of "                                                  , ;
         " Already defined. Program terminated"                  , ;
         "Browse: Type Not Allowed. Program terminated"          , ;
         "Browse: Append Clause Can't Be Used With Fields Not Belonging To Browse WorkArea.", ;
         "Record Is Being Edited By Another User"                , ;
         "Warning"                                               , ;
         "Invalid Entry"                                          }
      _HMG_BRWLangMessage := { 'Are you sure ?' , 'Delete Record' }

      // EDIT MESSAGES

      _HMG_aABMLangUser   := { Chr( 13 ) + "Delete record" + Chr( 13 ) + "Are you sure ?" + Chr( 13 )                  , ;
         Chr( 13 ) + "Index file missing" + Chr( 13 ) + "Can`t do search" + Chr( 13 )            , ;
         Chr( 13 ) + "Can`t find index field" + Chr( 13 ) + "Can`t do search" + Chr( 13 )        , ;
         Chr( 13 ) + "Can't do search by" + Chr( 13 ) + "fields memo or logic" + Chr( 13 )       , ;
         Chr( 13 ) + "Record not found" + Chr( 13 )                                        , ;
         Chr( 13 ) + "To many cols" + Chr( 13 ) + "The report can't fit in the sheet" + Chr( 13 ) }

      _HMG_aABMLangLabel  := { "Record"              , ;
         "Record count"        , ;
         "       (New)"        , ;
         "      (Edit)"        , ;
         "Enter record number" , ;
         "Find"                , ;
         "Search text"         , ;
         "Search date"         , ;
         "Search number"       , ;
         "Report definition"   , ;
         "Report columns"      , ;
         "Available columns"   , ;
         "Initial record"      , ;
         "Final record"        , ;
         "Report of "          , ;
         "Date:"               , ;
         "Initial record:"     , ;
         "Final record:"       , ;
         "Ordered by:"         , ;
         "Yes"                 , ;
         "No"                  , ;
         "Page "               , ;
         " of "                 }

      _HMG_aABMLangButton := { "Close"    , ;
         "New"      , ;
         "Edit"     , ;
         "Delete"   , ;
         "Find"     , ;
         "Goto"     , ;
         "Report"   , ;
         "First"    , ;
         "Previous" , ;
         "Next"     , ;
         "Last"     , ;
         "Save"     , ;
         "Cancel"   , ;
         "Add"      , ;
         "Remove"   , ;
         "Print"    , ;
         "Close"     }
      _HMG_aABMLangError  := { "EDIT, workarea name missing"          , ;
         "EDIT, this workarea has more than 16 fields"              , ;
         "EDIT, refresh mode out of range (please report bug)"      , ;
         "EDIT, main event number out of range (please report bug)" , ;
         "EDIT, list event number out of range (please report bug)"  }

      // EDIT EXTENDED MESSAGES

      _HMG_aLangButton := { ;
      "&Close",             ; // 1
      "&New",               ; // 2
      "&Modify",            ; // 3
      "&Delete",            ; // 4
      "&Find",              ; // 5
      "&Print",             ; // 6
      "&Cancel",            ; // 7
      "&Ok",                ; // 8
      "&Copy",              ; // 9
      "&Activate Filter",   ; // 10
      "&Deactivate Filter", ; // 11
      "&Restore",           ; // 12
		"Retry"               } // 13

      _HMG_aLangLabel := {            ;
         "None",                         ; // 1
      "Record",                       ; // 2
      "Total",                        ; // 3
      "Active order",                 ; // 4
      "Options",                      ; // 5
      "New record",                   ; // 6
      "Modify record",                ; // 7
      "Select record",                ; // 8
      "Find record",                  ; // 9
      "Print options",                ; // 10
      "Available fields",             ; // 11
      "Fields to print",              ; // 12
      "Available printers",           ; // 13
      "First record to print",        ; // 14
      "Last record to print",         ; // 15
      "Delete record",                ; // 16
      "Preview",                      ; // 17
      "View page thumbnails",         ; // 18
      "Filter Condition: ",           ; // 19
      "Filtered: ",                   ; // 20
      "Filtering Options" ,           ; // 21
      "Database Fields" ,             ; // 22
      "Comparison Operator",          ; // 23
      "Filter Value",                 ; // 24
      "Select Field To Filter",       ; // 25
      "Select Comparison Operator",   ; // 26
      "Equal",                        ; // 27
      "Not Equal",                    ; // 28
      "Greater Than",                 ; // 29
      "Lower Than",                   ; // 30
      "Greater or Equal Than",        ; // 31
      "Lower or Equal Than"           } // 32
      _HMG_aLangUser := { ;
         CRLF + "Can't find an active area.   "  + CRLF + "Please select any area before call EDIT   " + CRLF,       ; // 1
      "Type the field value (any text)",                                                                          ; // 2
      "Type the field value (any number)",                                                                        ; // 3
      "Select the date",                                                                                          ; // 4
      "Check for true value",                                                                                     ; // 5
      "Enter the field value",                                                                                    ; // 6
      "Select any record and press OK",                                                                           ; // 7
      CRLF + "You are going to delete the active record   " + CRLF + "Are you sure?    " + CRLF,                  ; // 8
      CRLF + "There isn't any active order   " + CRLF + "Please select one   " + CRLF,                            ; // 9
      CRLF + "Can't do searches by fields memo or logic   " + CRLF,                                               ; // 10
      CRLF + "Record not found   " + CRLF,                                                                        ; // 11
      "Select the field to include to list",                                                                      ; // 12
      "Select the field to exclude from list",                                                                    ; // 13
      "Select the printer",                                                                                       ; // 14
      "Push button to include field",                                                                             ; // 15
      "Push button to exclude field",                                                                             ; // 16
      "Push button to select the first record to print",                                                          ; // 17
      "Push button to select the last record to print",                                                           ; // 18
      CRLF + "No more fields to include   " + CRLF,                                                               ; // 19
      CRLF + "First select the field to include   " + CRLF,                                                       ; // 20
      CRLF + "No more fields to exlude   " + CRLF,                                                                ; // 21
      CRLF + "First select th field to exclude   " + CRLF,                                                        ; // 22
      CRLF + "You don't select any field   " + CRLF + "Please select the fields to include on print   " + CRLF,   ; // 23
      CRLF + "Too many fields   " + CRLF + "Reduce number of fields   " + CRLF,                                   ; // 24
      CRLF + "Printer not ready   " + CRLF,                                                                       ; // 25
      "Ordered by",                                                                                               ; // 26
      "From record",                                                                                              ; // 27
      "To record",                                                                                                ; // 28
      "Yes",                                                                                                      ; // 29
      "No",                                                                                                       ; // 30
      "Page:",                                                                                                    ; // 31
      CRLF + "Please select a printer   " + CRLF,                                                                 ; // 32
      "Filtered by",                                                                                              ; // 33
      CRLF + "There is an active filter    " + CRLF,                                                              ; // 34
      CRLF + "Can't filter by memo fields    " + CRLF,                                                            ; // 35
      CRLF + "Select the field to filter    " + CRLF,                                                             ; // 36
      CRLF + "Select any operator to filter    " + CRLF,                                                          ; // 37
      CRLF + "Type any value to filter    " + CRLF,                                                               ; // 38
      CRLF + "There isn't any active filter    " + CRLF,                                                          ; // 39
      CRLF + "Deactivate filter?   " + CRLF,                                                                      ; // 40
      CRLF + "Record locked by another user    " + CRLF,                                                          ; // 41
      CRLF + "You are going to restore the deleted record   " + CRLF + "Are you sure?    " + CRLF                 } // 42

   CASE cLang == "EU"  // Basque
      /////////////////////////////////////////////////////////////
      // BASQUE
      ////////////////////////////////////////////////////////////

      // MISC MESSAGES

      _HMG_MESSAGE [1] := 'Are you sure ?'
      _HMG_MESSAGE [2] := 'Close Window'
      _HMG_MESSAGE [3] := 'Close not allowed'
      _HMG_MESSAGE [4] := 'Program Already Running'
      _HMG_MESSAGE [5] := 'Edit'
      _HMG_MESSAGE [6] := 'Ok'
      _HMG_MESSAGE [7] := 'Cancel'
      _HMG_MESSAGE [8] := 'Apply'
      _HMG_MESSAGE [9] := 'Pag.'
      _HMG_MESSAGE [10] := 'Attention'
      _HMG_MESSAGE [11] := 'Information'
      _HMG_MESSAGE [12] := 'Stop'

      // BROWSE MESSAGES

      _HMG_BRWLangButton := { "Append"  , ;
         "Edit"    , ;
         "&Cancel"  , ;
         "&OK"       }
      _HMG_BRWLangError  := { "Window: "                                           , ;
         " is not defined. Program terminated"                   , ;
         "MiniGUI Error"                                         , ;
         "Control: "                                             , ;
         " Of "                                                  , ;
         " Already defined. Program terminated"                  , ;
         "Browse: Type Not Allowed. Program terminated"          , ;
         "Browse: Append Clause Can't Be Used With Fields Not Belonging To Browse WorkArea.", ;
         "Record Is Being Edited By Another User"                , ;
         "Warning"                                               , ;
         "Invalid Entry"                                          }
      _HMG_BRWLangMessage := { 'Are you sure ?' , 'Delete Record' }

      // EDIT MESSAGES

      _HMG_aABMLangUser   := { Chr( 13 ) + "Delete record" + Chr( 13 ) + "Are you sure ?" + Chr( 13 ), ;
         Chr( 13 ) + "Index file missing" + Chr( 13 ) + "Can`t do search" + Chr( 13 )            , ;
         Chr( 13 ) + "Can`t find index field" + Chr( 13 ) + "Can`t do search" + Chr( 13 )        , ;
         Chr( 13 ) + "Can't do search by" + Chr( 13 ) + "fields memo or logic" + Chr( 13 )       , ;
         Chr( 13 ) + "Record not found" + Chr( 13 )                                        , ;
         Chr( 13 ) + "To many cols" + Chr( 13 ) + "The report can't fit in the sheet" + Chr( 13 ) }

      _HMG_aABMLangLabel  := { "Record"              , ;
         "Record count"        , ;
         "       (New)"        , ;
         "      (Edit)"        , ;
         "Enter record number" , ;
         "Find"                , ;
         "Search text"         , ;
         "Search date"         , ;
         "Search number"       , ;
         "Report definition"   , ;
         "Report columns"      , ;
         "Available columns"   , ;
         "Initial record"      , ;
         "Final record"        , ;
         "Report of "          , ;
         "Date:"               , ;
         "Initial record:"     , ;
         "Final record:"       , ;
         "Ordered by:"         , ;
         "Yes"                 , ;
         "No"                  , ;
         "Page "               , ;
         " of "                 }

      _HMG_aABMLangButton := { "Close"    , ;
         "New"      , ;
         "Edit"     , ;
         "Delete"   , ;
         "Find"     , ;
         "Goto"     , ;
         "Report"   , ;
         "First"    , ;
         "Previous" , ;
         "Next"     , ;
         "Last"     , ;
         "Save"     , ;
         "Cancel"   , ;
         "Add"      , ;
         "Remove"   , ;
         "Print"    , ;
         "Close"     }
      _HMG_aABMLangError  := { "EDIT, workarea name missing"          , ;
         "EDIT, this workarea has more than 16 fields"              , ;
         "EDIT, refresh mode out of range (please report bug)"      , ;
         "EDIT, main event number out of range (please report bug)" , ;
         "EDIT, list event number out of range (please report bug)"  }

      // EDIT EXTENDED

      _HMG_aLangButton := {   ;
         "&Itxi",             ; // 1
         "&Berria",           ; // 2
         "&Aldatu",           ; // 3
         "&Ezabatu",          ; // 4
         "Bi&latu",           ; // 5
         "In&primatu",        ; // 6
         "&Utzi",             ; // 7
         "&Ok",               ; // 8
         "&Kopiatu",          ; // 9
         "I&ragazkia Ezarri", ; // 10
         "Ira&gazkia Kendu",  ; // 11
         "&Restore",          ; // 12
			"Retry"              } // 13

      _HMG_aLangLabel := {                  ;
         "Bat ere ez",                      ; // 1
      "Erregistroa",                        ; // 2
      "Guztira",                            ; // 3
      "Orden Aktiboa",                      ; // 4
      "Aukerak",                            ; // 5
      "Erregistro Berria",                  ; // 6
      "Erregistroa Aldatu",                 ; // 7
      "Erregistroa Aukeratu",               ; // 8
      "Erregistroa Bilatu",                 ; // 9
      "Inprimatze-aukerak",                 ; // 10
      "Eremu Libreak",                      ; // 11
      "Inprimatzeko Eremuak",               ; // 12
      "Inprimagailu Libreak",               ; // 13
      "Inprimatzeko Lehenengo Erregistroa", ; // 14
      "Inprimatzeko Azken Erregistroa",     ; // 15
      "Erregistroa Ezabatu",                ; // 16
      "Aurreikusi",                         ; // 17
      "Orrien Irudi Txikiak Ikusi",         ; // 18
      "Iragazkiaren Baldintza: ",           ; // 19
      "Iragazita: ",                        ; // 20
      "Iragazte-aukerak" ,                  ; // 21
      "Datubasearen Eremuak" ,              ; // 22
      "Konparaketa Eragilea",               ; // 23
      "Iragazkiaren Balioa",                ; // 24
      "Iragazteko Eremua Aukeratu",         ; // 25
      "Konparaketa Eragilea Aukeratu",      ; // 26
      "Berdin",                             ; // 27
      "Ezberdin",                           ; // 28
      "Handiago",                           ; // 29
      "Txikiago",                           ; // 30
      "Handiago edo Berdin",                ; // 31
      "Txikiago edo Berdin"                 } // 32
      _HMG_aLangUser := { ;
         CRLF + "Ezin da area aktiborik aurkitu.   "  + CRLF + "Mesedez aukeratu area EDIT deitu baino lehen   " + CRLF,  ; // 1
      "Eremuaren balioa idatzi (edozein testu)",                                                                       ; // 2
      "Eremuaren balioa idatzi (edozein zenbaki)",                                                                     ; // 3
      "Data aukeratu",                                                                                                 ; // 4
      "Markatu egiazko baliorako",                                                                                     ; // 5
      "Eremuaren balioa sartu",                                                                                        ; // 6
      "Edozein erregistro aukeratu eta OK sakatu",                                                                     ; // 7
      CRLF + "Erregistro aktiboa ezabatuko duzu   " + CRLF + "Ziur zaude?    " + CRLF,                                 ; // 8
      CRLF + "Ez dago orden aktiborik   " + CRLF + "Mesedez aukeratu bat   " + CRLF,                                   ; // 9
      CRLF + "Memo edo eremu logikoen arabera ezin bilaketarik egin   " + CRLF,                                        ; // 10
      CRLF + "Erregistroa ez da aurkitu   " + CRLF,                                                                    ; // 11
      "Zerrendan sartzeko eremua aukeratu",                                                                            ; // 12
      "Zerrendatik kentzeko eremua aukeratu",                                                                          ; // 13
      "Inprimagailua aukeratu",                                                                                        ; // 14
      "Sakatu botoia eremua sartzeko",                                                                                 ; // 15
      "Sakatu botoia eremua kentzeko",                                                                                 ; // 16
      "Sakatu botoia inprimatzeko lehenengo erregistroa aukeratzeko",                                                  ; // 17
      "Sakatu botoia inprimatzeko azken erregistroa aukeratzeko",                                                      ; // 18
      CRLF + "Sartzeko eremu gehiagorik ez   " + CRLF,                                                                 ; // 19
      CRLF + "Lehenago aukeratu sartzeko eremua   " + CRLF,                                                            ; // 20
      CRLF + "Kentzeko eremu gehiagorik ez   " + CRLF,                                                                 ; // 21
      CRLF + "Lehenago aukeratu kentzeko eremua   " + CRLF,                                                            ; // 22
      CRLF + "Ez duzu eremurik aukeratu  " + CRLF + "Mesedez aukeratu inprimaketan sartzeko eremuak   " + CRLF,        ; // 23
      CRLF + "Eremu gehiegi   " + CRLF + "Murriztu eremu kopurua   " + CRLF,                                           ; // 24
      CRLF + "Inprimagailua ez dago prest   " + CRLF,                                                                  ; // 25
      "Ordenatuta honen arabera:",                                                                                     ; // 26
      "Erregistro honetatik:",                                                                                         ; // 27
      "Erregistro honetara:",                                                                                          ; // 28
      "Bai",                                                                                                           ; // 29
      "Ez",                                                                                                            ; // 30
      "Orrialdea:",                                                                                                    ; // 31
      CRLF + "Mesedez aukeratu inprimagailua   " + CRLF,                                                               ; // 32
      "Iragazita honen arabera:",                                                                                      ; // 33
      CRLF + "Iragazki aktiboa dago    " + CRLF,                                                                       ; // 34
      CRLF + "Ezin iragazi Memo eremuen arabera    " + CRLF,                                                           ; // 35
      CRLF + "Iragazteko eremua aukeratu    " + CRLF,                                                                  ; // 36
      CRLF + "Iragazteko edozein eragile aukeratu    " + CRLF,                                                         ; // 37
      CRLF + "Idatzi edozein balio iragazteko    " + CRLF,                                                             ; // 38
      CRLF + "Ez dago iragazki aktiborik    " + CRLF,                                                                  ; // 39
      CRLF + "Iragazkia kendu?   " + CRLF,                                                                             ; // 40
      CRLF + "Record locked by another user    " + CRLF,                                                               ; // 41
      CRLF + "You are going to restore the deleted record   " + CRLF + "Are you sure?    " + CRLF                      } // 42

   CASE cLang == "FR"  // French
      /////////////////////////////////////////////////////////////
      // FRENCH
      ////////////////////////////////////////////////////////////

      // MISC MESSAGES

      _HMG_MESSAGE [1] := 'Etes-vous sre ?'
      _HMG_MESSAGE [2] := 'Fermer la fentre'
      _HMG_MESSAGE [3] := 'Fermeture interdite'
      _HMG_MESSAGE [4] := 'Programme dj activ'
      _HMG_MESSAGE [5] := 'Editer'
      _HMG_MESSAGE [6] := 'Ok'
      _HMG_MESSAGE [7] := 'Abandonner'
      _HMG_MESSAGE [8] := 'Apply'
      _HMG_MESSAGE [9] := 'Pag.'
      _HMG_MESSAGE [10] := 'Attention'
      _HMG_MESSAGE [11] := 'Information'
      _HMG_MESSAGE [12] := 'Stop'

      // BROWSE

      _HMG_BRWLangButton := { "Ajout"         , ;
         "Modification"  , ;
         "Annuler"       , ;
         "OK"             }
      _HMG_BRWLangError  := { "Fentre: "                                             , ;
         " n'est pas dfinie. Programme termin"                 , ;
         "Erreur MiniGUI"                                        , ;
         "Contrle: "                                            , ;
         " De "                                                  , ;
         " Dj dfini. Programme termin"                       , ;
         "Modification: Type non autoris. Programme termin"    , ;
         "Modification: La clause Ajout ne peut tre utilise avec des champs n'appartenant pas  la zone de travail de Modification. Programme termin", ;
         "L'enregistrement est utilis par un autre utilisateur"  , ;
         "Erreur"                                                , ;
         "Entre invalide"                                        }
      _HMG_BRWLangMessage := { 'Etes-vous sre ?' , 'Enregistrement dtruit' }

      // EDIT

      _HMG_aABMLangUser   := { Chr( 13 ) + "Suppression d'enregistrement" + Chr( 13 ) + "Etes-vous sre ?" + Chr( 13 )  , ;
         Chr( 13 ) + "Index manquant" + Chr( 13 ) + "Recherche impossible" + Chr( 13 )            , ;
         Chr( 13 ) + "Champ Index introuvable" + Chr( 13 ) + "Recherche impossible" + Chr( 13 )   , ;
         Chr( 13 ) + "Recherche impossible" + Chr( 13 ) + "sur champs memo ou logique" + Chr( 13 ), ;
         Chr( 13 ) + "Enregistrement non trouv" + Chr( 13 )                                                     , ;
         Chr( 13 ) + "Trop de colonnes" + Chr( 13 ) + "L'tat ne peut tre imprim" + Chr( 13 )      }
      _HMG_aABMLangLabel  := { "Enregistrement"                       , ;
         "Nb. total enr."                       , ;
         "   (Ajouter)"                        , ;
         "  (Modifier)"                        , ;
         "Entrez le numro de l'enregistrement" , ;
         "Trouver"                              , ;
         "Chercher texte"                       , ;
         "Chercher date"                        , ;
         "Chercher numro"                      , ;
         "Dfinition de l'tat"                 , ;
         "Colonnes de l'tat"                   , ;
         "Colonnes disponibles"                 , ;
         "Enregistrement de dbut"              , ;
         "Enregistrement de fin"                , ;
         "Etat de "                             , ;
         "Date:"                                , ;
         "Enregistrement de dbut:"             , ;
         "Enregistrement de fin:"               , ;
         "Tri par:"                            , ;
         "Oui"                                  , ;
         "Non"                                  , ;
         " Page"                                , ;
         " de "                                 }
      _HMG_aABMLangButton := { "Fermer"      , ;
         "Nouveau"     , ;
         "Modifier"    , ;
         "Supprimer"   , ;
         "Trouver"     , ;
         "Aller "     , ;
         "Etat"   , ;
         "Premier"     , ;
         "Prcdent"   , ;
         "Suivant"     , ;
         "Dernier"     , ;
         "Enregistrer" , ;
         "Annuler"     , ;
         "Ajouter"     , ;
         "Retirer"     , ;
         "Imprimer"    , ;
         "Fermer"      }
      _HMG_aABMLangError  := { "EDIT, nom de la table manquant"                   , ;
         "EDIT, la table a plus de 16 champs"                                     , ;
         "EDIT, mode rafraichissement hors limite (Rapport d'erreur merci)"       , ;
         "EDIT, vnement principal nombre hors limite (Rapport d'erreur merci)"  , ;
         "EDIT, liste d'vnements nombre hors limite (Rapport d'erreur merci)"   }

      // EDIT EXTENDED

      _HMG_aLangButton := {  ;
         "&Fermer",          ; // 1
         "&Nouveau",         ; // 2
         "&Modifier",        ; // 3
         "&Supprimer",       ; // 4
         "&Trouver",         ; // 5
         "&Imprimer",        ; // 6
         "&Abandon",         ; // 7
         "&Ok",              ; // 8
         "&Copier",          ; // 9
         "&Activer Filtre",  ; // 10
         "&Dactiver Filtre",; // 11
         "&Reconstituer",    ; // 12
			"Retry"             } // 13

      _HMG_aLangLabel := {                       ;
         "Aucun",                                ; // 1
      "Enregistrement",                          ; // 2
      "Total",                                   ; // 3
      "Ordre actif",                             ; // 4
      "Options",                                 ; // 5
      "Nouvel enregistrement",                   ; // 6
      "Modifier enregistrement",                 ; // 7
      "Selectionner enregistrement",             ; // 8
      "Trouver enregistrement",                  ; // 9
      "Imprimer options",                        ; // 10
      "Champs disponibles",                      ; // 11
      "Champs  imprimer",                       ; // 12
      "Imprimantes connectes",                  ; // 13
      "Premier enregistrement  imprimer",       ; // 14
      "Dernier enregistrement  imprimer",       ; // 15
      "Enregistrement supprim",                 ; // 16
      "Prvisualisation",                        ; // 17
      "Aperu pages",                            ; // 18
      "Condition filtre : ",                     ; // 19
      "Filtr : ",                               ; // 20
      "Options de filtrage" ,                    ; // 21
      "Champs de la Bdd" ,                       ; // 22
      "Oprateurs de comparaison",               ; // 23
      "Valeur du filtre",                        ; // 24
      "Selectionner le champ  filtrer",         ; // 25
      "Selectionner l'oprateur de comparaison", ; // 26
      "Egal",                                    ; // 27
      "Diffrent",                               ; // 28
      "Plus grand",                              ; // 29
      "Plus petit",                              ; // 30
      "Plus grand ou gal",                      ; // 31
      "Plus petit ou gal"                       } // 32
      _HMG_aLangUser := { ;
         CRLF + "Ne peut trouver une base active.   "  + CRLF + "Slectionner une base avant la fonction EDIT  " + CRLF,            ; // 1
      "Entrer la valeur du champ (du texte)",                                                                                       ; // 2
      "Entrer la valeur du champ (un nombre)",                                                                                      ; // 3
      "Slectionner la date",                                                                                                       ; // 4
      "Vrifier la valeur logique",                                                                                                 ; // 5
      "Entrer la valeur du champ",                                                                                                  ; // 6
      "Slectionner un enregistrement et appuyer sur OK",                                                                           ; // 7
      CRLF + "Vous voulez dtruire l'enregistrement actif  " + CRLF + "Etes-vous sre?   " + CRLF,                                  ; // 8
      CRLF + "Il n'y a pas d'ordre actif   " + CRLF + "Slectionner en un   " + CRLF,                                               ; // 9
      CRLF + "Ne peut faire de recherche sur champ memo ou logique   " + CRLF,                                                      ; // 10
      CRLF + "Enregistrement non trouv  " + CRLF,                                                                                  ; // 11
      "Slectionner le champ  inclure  la liste",                                                                                 ; // 12
      "Slectionner le champ  exclure de la liste",                                                                                ; // 13
      "Slectionner l'imprimante",                                                                                                  ; // 14
      "Appuyer sur le bouton pour inclure un champ",                                                                                ; // 15
      "Appuyer sur le bouton pour exclure un champ",                                                                                ; // 16
      "Appuyer sur le bouton pour slectionner le premier enregistrement  imprimer",                                               ; // 17
      "Appuyer sur le bouton pour slectionner le dernier champ  imprimer",                                                        ; // 18
      CRLF + "Plus de champs  inclure   " + CRLF,                                                                                  ; // 19
      CRLF + "Slectionner d'abord les champs  inclure   " + CRLF,                                                                 ; // 20
      CRLF + "Plus de champs  exclure   " + CRLF,                                                                                  ; // 21
      CRLF + "Slectionner d'abord les champs  exclure   " + CRLF,                                                                 ; // 22
      CRLF + "Vous n'avez slectionn aucun champ   " + CRLF + "Slectionner les champs  inclure dans l'impression   " + CRLF,     ; // 23
      CRLF + "Trop de champs   " + CRLF + "Rduiser le nombre de champs   " + CRLF,                                                 ; // 24
      CRLF + "Imprimante pas prte   " + CRLF,                                                                                      ; // 25
      "Tri par",                                                                                                                   ; // 26
      "De l'enregistrement",                                                                                                        ; // 27
      "A l'enregistrement",                                                                                                         ; // 28
      "Oui",                                                                                                                        ; // 29
      "Non",                                                                                                                        ; // 30
      "Page:",                                                                                                                      ; // 31
      CRLF + "Slectionner une imprimante   " + CRLF,                                                                               ; // 32
      "Filtr par",                                                                                                                 ; // 33
      CRLF + "Il y a un filtre actif    " + CRLF,                                                                                   ; // 34
      CRLF + "Filtre impossible sur champ memo    " + CRLF,                                                                         ; // 35
      CRLF + "Slectionner un champ de filtre    " + CRLF,                                                                          ; // 36
      CRLF + "Slectionner un oprateur de filtre   " + CRLF,                                                                       ; // 37
      CRLF + "Entrer une valeur au filtre    " + CRLF,                                                                              ; // 38
      CRLF + "Il n'y a aucun filtre actif    " + CRLF,                                                                              ; // 39
      CRLF + "Dsactiver le filtre?   " + CRLF,                                                                                     ; // 40
      CRLF + "Record locked by another user    " + CRLF,                                                                            ; // 41
      CRLF + "You are going to restore the deleted record   " + CRLF + "Are you sure?    " + CRLF                                   } // 42

   CASE cLang == "DE"  // German
      /////////////////////////////////////////////////////////////
      // GERMAN
      ////////////////////////////////////////////////////////////

      // MISC MESSAGES

      _HMG_MESSAGE [1] := 'Sind Sie sicher ?'
      _HMG_MESSAGE [2] := 'Fenster schlieen'
      _HMG_MESSAGE [3] := 'Schlieen nicht erlaubt'
      _HMG_MESSAGE [4] := 'Programm luft bereits'
      _HMG_MESSAGE [5] := 'Bearbeiten'
      _HMG_MESSAGE [6] := 'OK'
      _HMG_MESSAGE [7] := 'Abbruch'
      _HMG_MESSAGE [8] := 'Anwenden'
      _HMG_MESSAGE [9] := 'Seite'
      _HMG_MESSAGE [10] := 'Warnung'
      _HMG_MESSAGE [11] := 'Information'
      _HMG_MESSAGE [12] := 'Stop'

      // BROWSE

      _HMG_BRWLangButton := { "Anhngen"  , ;
         "Bearbeiten"    , ;
         "&Abbruch"  , ;
         "&OK"       }
      _HMG_BRWLangError  := { "Window: "                         , ;
         " ist nicht definiert. Programm Abbruch"                , ;
         "MiniGUI Error"                                         , ;
         "Control: "                                             , ;
         " Of "                                                  , ;
         " bereits definiert. Programm Abbruch"                  , ;
         "Browse: Type nicht erlaubt. Programm Abbruch"          , ;
         "Browse: Append kann nicht auf Feldern in anderen Workarea angewendet werden.", ;
         "Datensatz in Bearbeitung eines anderen Nutzers"        , ;
         "Warnung"                                               , ;
         "Ungltiger Eintrag"                                          }
      _HMG_BRWLangMessage := { 'Sind Sie sicher ?' , 'Datensatz lschen' }

      // EDIT

      _HMG_aABMLangUser   := { Chr( 13 ) + "Datensatz lschen" + Chr( 13 ) + "Sind Sie sicher ?" + Chr( 13 ), ;
         Chr( 13 ) + "Index Datei fehlt" + Chr( 13 ) + "Suche nicht mglich" + Chr( 13 ), ;
         Chr( 13 ) + "Finde Indexdatenfeld nicht" + Chr( 13 ) + "Suche nicht mglich" + Chr( 13 ), ;
         Chr( 13 ) + "Suche in Memo oder Logic " + Chr( 13 ) + "Feld nicht mglich" + Chr( 13 ), ;
         Chr( 13 ) + "Datensatz nicht gefunden" + Chr( 13 ), ;
         Chr( 13 ) + "Zu viele Spalten" + Chr( 13 ) + " Report passt nicht auf die Seite" + Chr( 13 ) }

      _HMG_aABMLangLabel  := { "Datensatz"              , ;
         "Datensatz Anzahl"        , ;
         "Datensatz (Neu)"        , ;
         "Datensatz (Edit)"        , ;
         "Datensatznummer eintragen" , ;
         "Suchen"                , ;
         "Suche Text"         , ;
         "Suche Datum"         , ;
         "Suche Nummer"       , ;
         "Report Definition"   , ;
         "Report Spalten"      , ;
         "Verfgbare Spalten"     , ;
         "Erster Datensatz"      , ;
         "Letzter Datensatz"        , ;
         "Report vom "          , ;
         "Datum:"               , ;
         "Erster Datensatz:"     , ;
         "Letzter Datensatz:"       , ;
         "Sortieren nach:"         , ;
         "Ja"                 , ;
         "Nein"                  , ;
         "Seite "               , ;
         " von "                 }
      _HMG_aABMLangButton := { "Schlieen"    , ;
         "Neu"      , ;
         "Bearbeiten"     , ;
         "Lschen"   , ;
         "Suchen"     , ;
         "Gehe zu"     , ;
         "Report"   , ;
         "Erster"    , ;
         "Davor" , ;
         "Nchster"     , ;
         "Letzter"     , ;
         "Speichern"     , ;
         "Abbrechen"   , ;
         "Hinzufgen"      , ;
         "Entfernen"   , ;
         "Drucken"    , ;
         "Schlieen"     }
      _HMG_aABMLangError  := { "EDIT, Workarea Name fehlt" , ;
         "EDIT, Workarea hat mehr als 16 Felder" , ;
         "EDIT, Aktualisierung ausserhalb des Bereichs (siehe Fehlermeldungen)" , ;
         "EDIT, Haupt Ereignis ausserhalb des Bereichs (siehe Fehlermeldungen)" , ;
         "EDIT, Listen Ereignis ausserhalb des Bereichs (siehe Fehlermeldungen)"  }

      // EDIT EXTENDED

      _HMG_aLangButton := {  ;
      "S&chlieen",          ; // 1
      "&Neu",                ; // 2
      "&Bearbeiten",         ; // 3
      "&Lschen",            ; // 4
      "&Suchen",             ; // 5
      "&Drucken",            ; // 6
      "&Abbruch",            ; // 7
      "&Ok",                 ; // 8
      "&Kopieren",           ; // 9
      "&Filter aktivieren",  ; // 10
      "&Filter deaktivieren",; // 11
      "&Wiederherstellen",   ; // 12
		"Retry"                } // 13

      _HMG_aLangLabel := { ;
      "Keine",                                         ; // 1
      "Datensatz",                                     ; // 2
      "Gesamt",                                        ; // 3
      "Aktive Sortierung",                             ; // 4
      "Einstellungen",                                 ; // 5
      "Neuer Datensatz",                               ; // 6
      "Datensatz bearbeiten",                          ; // 7
      "Datensatz auswhlen",                           ; // 8
      "Datensatz finden",                              ; // 9
      "Druckeinstellungen",                            ; // 10
      "Verfgbare Felder",                             ; // 11
      "Zu druckende Felder",                           ; // 12
      "Verfgbare Drucker",                            ; // 13
      "Erster zu druckender Datensatz",                ; // 14
      "Letzter zu druckender Datensatz",               ; // 15
      "Datensatz lschen",                             ; // 16
      "Vorschau",                                      ; // 17
      "bersicht",                                     ; // 18
      "Filterbedingung: ",                             ; // 19
      "Gefiltert: ",                                   ; // 20
      "Filter-Einstellungen" ,                         ; // 21
      "Datenbank-Felder" ,                             ; // 22
      "Vergleichs-Operator",                           ; // 23
      "Filterwert",                                    ; // 24
      "Zu filterndes Feld auswhlen",                  ; // 25
      "Vergleichs-Operator auswhlen",                 ; // 26
      "Gleich",                                        ; // 27
      "Ungleich",                                      ; // 28
      "Grer als",                                    ; // 29
      "Kleiner als",                                   ; // 30
      "Grer oder gleich als",                        ; // 31
      "Kleiner oder gleich als"                        } // 32
      _HMG_aLangUser := { ;
         CRLF + "Kein aktiver Arbeitsbereich gefunden.   "  + CRLF + "Bitte einen Arbeitsbereich auswhlen vor dem Aufruf von EDIT   " + CRLF,    ; // 1
      "Einen Text eingeben (alphanumerisch)",                                                                                                     ; // 2
      "Eine Zahl eingeben",                                                                                                                       ; // 3
      "Datum auswhlen",                                                                                                                          ; // 4
      "Fr positive Auswahl einen Haken setzen",                                                                                                  ; // 5
      "Einen Text eingeben (alphanumerisch)",                                                                                                     ; // 6
      "Einen Datensatz whlen und mit OK besttigen",                                                                                             ; // 7
      CRLF + "Sie sind im Begriff, den aktiven Datensatz zu lschen.   " + CRLF + "Sind Sie sicher?    " + CRLF,                                  ; // 8
      CRLF + "Es ist keine Sortierung aktiv.   " + CRLF + "Bitte whlen Sie eine Sortierung   " + CRLF,                                           ; // 9
      CRLF + "Suche nach den Feldern memo oder logisch nicht mglich.   " + CRLF,                                                                 ; // 10
      CRLF + "Datensatz nicht gefunden   " + CRLF,                                                                                                ; // 11
      "Bitte ein Feld zum Hinzufgen zur Liste whlen",                                                                                           ; // 12
      "Bitte ein Feld zum Entfernen aus der Liste whlen ",                                                                                       ; // 13
      "Drucker auswhlen",                                                                                                                        ; // 14
      "Schaltflche  Feld hinzufgen",                                                                                                            ; // 15
      "Schaltflche  Feld Entfernen",                                                                                                             ; // 16
      "Schaltflche  Auswahl erster zu druckender Datensatz",                                                                                     ; // 17
      "Schaltflche  Auswahl letzte zu druckender Datensatz",                                                                                     ; // 18
      CRLF + "Keine Felder zum Hinzufgen mehr vorhanden   " + CRLF,                                                                              ; // 19
      CRLF + "Bitte erst ein Feld zum Hinzufgen whlen   " + CRLF,                                                                               ; // 20
      CRLF + "Keine Felder zum Entfernen vorhanden   " + CRLF,                                                                                    ; // 21
      CRLF + "Bitte ein Feld zum Entfernen whlen   " + CRLF,                                                                                     ; // 22
      CRLF + "Kein Feld ausgewhlt   " + CRLF + "Bitte die Felder fr den Ausdruck auswhlen   " + CRLF,                                          ; // 23
      CRLF + "Zu viele Felder   " + CRLF + "Bitte Anzahl der Felder reduzieren   " + CRLF,                                                        ; // 24
      CRLF + "Drucker nicht bereit   " + CRLF,                                                                                                    ; // 25
      "Sortiert nach",                                                                                                                            ; // 26
      "Von Datensatz",                                                                                                                            ; // 27
      "Bis Datensatz",                                                                                                                            ; // 28
      "Ja",                                                                                                                                       ; // 29
      "Nein",                                                                                                                                     ; // 30
      "Seite:",                                                                                                                                   ; // 31
      CRLF + "Bitte einen Drucker whlen   " + CRLF,                                                                                              ; // 32
      "Filtern nach",                                                                                                                             ; // 33
      CRLF + "Es ist kein aktiver Filter vorhanden    " + CRLF,                                                                                   ; // 34
      CRLF + "Kann nicht nach Memo-Feldern filtern    " + CRLF,                                                                                   ; // 35
      CRLF + "Feld zum Filtern auswhlen    " + CRLF,                                                                                             ; // 36
      CRLF + "Einen Operator zum Filtern auswhlen    " + CRLF,                                                                                   ; // 37
      CRLF + "Bitte einen Wert fr den Filter angeben    " + CRLF,                                                                                ; // 38
      CRLF + "Es ist kein aktiver Filter vorhanden    " + CRLF,                                                                                   ; // 39
      CRLF + "Filter deaktivieren?   " + CRLF,                                                                                                    ; // 40
      CRLF + "Datensatz gesperrt durch anderen Benutzer    " + CRLF,                                                                              ; // 41
      CRLF + "Gelschten Datensatz wiederherstellen   " + CRLF + "Sind sie sicher?    " + CRLF                                                    } // 42

   CASE cLang == "IT"  // Italian
      /////////////////////////////////////////////////////////////
      // ITALIAN
      ////////////////////////////////////////////////////////////

      // MISC MESSAGES

      _HMG_MESSAGE [1] := 'Sei sicuro ?'
      _HMG_MESSAGE [2] := 'Chiudi la finestra'
      _HMG_MESSAGE [3] := 'Chiusura non consentita'
      _HMG_MESSAGE [4] := 'Il programma  gi in esecuzione'
      _HMG_MESSAGE [5] := 'Edita'
      _HMG_MESSAGE [6] := 'Conferma'
      _HMG_MESSAGE [7] := 'Annulla'
      _HMG_MESSAGE [8] := 'Applica'
      _HMG_MESSAGE [9] := 'Pag.'
      _HMG_MESSAGE [10] := 'Attention'
      _HMG_MESSAGE [11] := 'Information'
      _HMG_MESSAGE [12] := 'Stop'

      // BROWSE

      _HMG_BRWLangButton := { "Aggiungere"   , ;
         "Modificare"  , ;
         "Cancellare"  , ;
         "OK"           }
      _HMG_BRWLangError  := { "Window: " , ;
         " non  definita. Programma terminato" , ;
         "Errore MiniGUI"  , ;
         "Controllo: " , ;
         " Di " , ;
         " Gi definito. Programma Terminato" , ;
         "Browse: Tipo non valido. Programma Terminato"  , ;
         "Browse: Modifica non possibile: il campo non  pertinente l'area di lavoro.Programma Terminato", ;
         "Record gi utilizzato da altro utente"                 , ;
         "Attenzione!"                                           , ;
         "Dato non valido" }
      _HMG_BRWLangMessage := { 'Sei sicuro ?' , 'Cancella Record' }

      // EDIT

      _HMG_aABMLangUser   := { Chr( 13 ) + "Cancellare il record" + Chr( 13 ) + "Sei sicuro ?" + Chr( 13 )      , ;
         Chr( 13 ) + "File indice mancante" + Chr( 13 ) + "Ricerca impossibile" + Chr( 13 )   , ;
         Chr( 13 ) + "Campo indice mancante" + Chr( 13 ) + "Ricerca impossibile" + Chr( 13 )  , ;
         Chr( 13 ) + "Ricerca impossibile per" + Chr( 13 ) + "campi memo o logici" + Chr( 13 ), ;
         Chr( 13 ) + "Record non trovato" + Chr( 13 )                                   , ;
         Chr( 13 ) + "Troppe colonne" + Chr( 13 ) + "Il report non pu essere stampato" + Chr( 13 ) }
      _HMG_aABMLangLabel  := { "Record"              , ;
         "Record totali"       , ;
         "  (Aggiungi)"        , ;
         "     (Nuovo)"        , ;
         "Inserire il numero del record" , ;
         "Ricerca"                , ;
         "Testo da cercare"         , ;
         "Data da cercare"         , ;
         "Numero da cercare"       , ;
         "Definizione del report"   , ;
         "Colonne del report"      , ;
         "Colonne totali"     , ;
         "Record Iniziale"      , ;
         "Record Finale"        , ;
         "Report di "          , ;
         "Data:"               , ;
         "Primo Record:"     , ;
         "Ultimo Record:"       , ;
         "Ordinare per:"         , ;
         "S"                 , ;
         "No"                  , ;
         "Pagina "               , ;
         " di "                 }
      _HMG_aABMLangButton := { "Chiudi"    , ;
         "Nuovo"      , ;
         "Modifica"     , ;
         "Cancella"   , ;
         "Ricerca"     , ;
         "Vai a"     , ;
         "Report"   , ;
         "Primo"    , ;
         "Precedente" , ;
         "Successivo"     , ;
         "Ultimo"     , ;
         "Salva"     , ;
         "Annulla"   , ;
         "Aggiungi"      , ;
         "Rimuovi"   , ;
         "Stampa"    , ;
         "Chiudi"     }
      _HMG_aABMLangError  := { "EDIT, il nome dell'area  mancante" , ;
         "EDIT, quest'area contiene pi di 16 campi" , ;
         "EDIT, modalit aggiornamento fuori dal limite (segnalare l'errore)" , ;
         "EDIT, evento pricipale fuori dal limite (segnalare l'errore)" , ;
         "EDIT, lista eventi fuori dal limite (segnalare l'errore)"  }

      // EDIT EXTENDED

      _HMG_aLangButton := {  ;
         "&Chiudi",          ; // 1
         "&Nuovo",           ; // 2
         "&Modifica",        ; // 3
         "&Elimina",         ; // 4
         "&Trova",           ; // 5
         "&Stampa",          ; // 6
         "&Annulla",         ; // 7
         "&Ok",              ; // 8
         "C&opia",           ; // 9
         "A&ttiva Filtro",   ; // 10
         "&Disattiva Filtro",; // 11
         "&Ripristina",      ; // 12
			"Retry"             } // 13

      _HMG_aLangLabel := {                ;
         "Nessuno",                       ; // 1
      "Record",                           ; // 2
      "Totale",                           ; // 3
      "Ordinamento attivo",               ; // 4
      "Opzioni",                          ; // 5
      "Nuovo record",                     ; // 6
      "Modifica record",                  ; // 7
      "Seleziona record",                 ; // 8
      "Trova record",                     ; // 9
      "Stampa opzioni",                   ; // 10
      "Campi disponibili",                ; // 11
      "Campi da stampare",                ; // 12
      "Stampanti disponibili",            ; // 13
      "Primo  record da stampare",        ; // 14
      "Ultimo record da stampare",        ; // 15
      "Cancella record",                  ; // 16
      "Anteprima",                        ; // 17
      "Visualizza pagina miniature",      ; // 18
      "Condizioni Filtro: ",              ; // 19
      "Filtrato: ",                       ; // 20
      "Opzioni Filtro" ,                  ; // 21
      "Campi del Database" ,              ; // 22
      "Operatori di comparazione",        ; // 23
      "Valore Filtro",                    ; // 24
      "Seleziona campo da filtrare",      ; // 25
      "Seleziona operatore comparazione", ; // 26
      "Uguale",                           ; // 27
      "Non Uguale",                       ; // 28
      "Maggiore di",                      ; // 29
      "Minore di",                        ; // 30
      "Maggiore o uguale a",              ; // 31
      "Minore o uguale a"                 } // 32
      _HMG_aLangUser := { ;
         CRLF + "Nessuna area attiva.   "  + CRLF + "Selezionare un'area prima della chiamata a EDIT   " + CRLF,  ; // 1
      "Digitare valore campo (testo)",                                                                         ; // 2
      "Digitare valore campo (numerico)",                                                                      ; // 3
      "Selezionare data",                                                                                      ; // 4
      "Attivare per valore TRUE",                                                                              ; // 5
      "Inserire valore campo",                                                                                 ; // 6
      "Seleziona un record and premi OK",                                                                      ; // 7
      CRLF + "Cancellazione record attivo   " + CRLF + "Sei sicuro?      " + CRLF,                             ; // 8
      CRLF + "Nessun ordinamento attivo     " + CRLF + "Selezionarne uno " + CRLF,                             ; // 9
      CRLF + "Ricerca non possibile su campi MEMO o LOGICI   " + CRLF,                                         ; // 10
      CRLF + "Record non trovato   " + CRLF,                                                                   ; // 11
      "Seleziona campo da includere nel listato",                                                              ; // 12
      "Seleziona campo da escludere dal listato",                                                              ; // 13
      "Selezionare la stampante",                                                                              ; // 14
      "Premi per includere il campo",                                                                          ; // 15
      "Premi per escludere il campo",                                                                          ; // 16
      "Premi per selezionare il primo record da stampare",                                                     ; // 17
      "Premi per selezionare l'ultimo record da stampare",                                                     ; // 18
      CRLF + "Nessun altro campo da inserire   " + CRLF,                                                       ; // 19
      CRLF + "Prima seleziona il campo da includere " + CRLF,                                                  ; // 20
      CRLF + "Nessun altro campo da escludere       " + CRLF,                                                  ; // 21
      CRLF + "Prima seleziona il campo da escludere " + CRLF,                                                  ; // 22
      CRLF + "Nessun campo selezionato     " + CRLF + "Selezionare campi da includere nel listato   " + CRLF,  ; // 23
      CRLF + "Troppi campi !   " + CRLF + "Redurre il numero di campi   " + CRLF,                              ; // 24
      CRLF + "Stampante non pronta..!   " + CRLF,                                                              ; // 25
      "Ordinato per",                                                                                          ; // 26
      "Dal record",                                                                                            ; // 27
      "Al  record",                                                                                            ; // 28
      "Si",                                                                                                    ; // 29
      "No",                                                                                                    ; // 30
      "Pagina:",                                                                                               ; // 31
      CRLF + "Selezionare una stampante   " + CRLF,                                                            ; // 32
      "Filtrato per ",                                                                                         ; // 33
      CRLF + "Esiste un filtro attivo     " + CRLF,                                                            ; // 34
      CRLF + "Filtro non previsto per campi MEMO   " + CRLF,                                                   ; // 35
      CRLF + "Selezionare campo da filtrare        " + CRLF,                                                   ; // 36
      CRLF + "Selezionare un OPERATORE per filtro  " + CRLF,                                                   ; // 37
      CRLF + "Digitare un valore per filtro        " + CRLF,                                                   ; // 38
      CRLF + "Nessun filtro attivo    " + CRLF,                                                                ; // 39
      CRLF + "Disattivare filtro ?   " + CRLF,                                                                 ; // 40
      CRLF + "Record bloccato da altro utente" + CRLF,                                                         ; // 41
      CRLF + "Ripristinare il record cancellato             " + CRLF + "Sei sicuro  ?    " + CRLF              } // 42

   CASE cLang == "PL"  // Polish
      /////////////////////////////////////////////////////////////
      // POLISH
      ////////////////////////////////////////////////////////////

      // MISC MESSAGES

      _HMG_MESSAGE [1] := 'Czy jeste pewny ?'
      _HMG_MESSAGE [2] := 'Zamknij okno'
      _HMG_MESSAGE [3] := 'Zamknicie niedozwolone'
      _HMG_MESSAGE [4] := 'Program ju uruchomiony'
      _HMG_MESSAGE [5] := 'Edycja'
      _HMG_MESSAGE [6] := 'Ok'
      _HMG_MESSAGE [7] := 'Porzu'
      _HMG_MESSAGE [8] := 'Zastosuj'
      _HMG_MESSAGE [9] := 'Str.'
      _HMG_MESSAGE [10] := 'Attention'
      _HMG_MESSAGE [11] := 'Information'
      _HMG_MESSAGE [12] := 'Stop'

      // BROWSE

      _HMG_BRWLangButton := { "Dodaj"   , ;
         "Edycja"     , ;
         "Porzu"   , ;
         "OK"        }
      _HMG_BRWLangError  := { "Okno: "                                              , ;
         " nie zdefiniowane.Program zakoczony"         , ;
         "Bd MiniGUI"                                         , ;
         "Kontrolka: "                                             , ;
         " z "                                                  , ;
         " ju zdefiniowana. Program zakoczony"                  , ;
         "Browse: Niedozwolony typ danych. Program zakoczony"          , ;
         "Browse: Klauzula Append nie moe by stosowana do pl nie nalecych do aktualnego obszaru roboczego. Program zakoczony", ;
         "Rekord edytowany przez innego uytkownika"                , ;
         "Ostrzeenie"                                               , ;
         "Nieprawidowy wpis"                                          }
      _HMG_BRWLangMessage := { 'Czy jeste pewny ?' , 'Skasuj rekord' }

      // EDIT

      _HMG_aABMLangUser   := { Chr( 13 ) + "Usunicie rekordu" + Chr( 13 ) + "Jeste pewny ?" + Chr( 13 )                 , ;
         Chr( 13 ) + "Bdny zbir indeksowy" + Chr( 13 ) + "Nie mona szuka" + Chr( 13 )         , ;
         Chr( 13 ) + "Nie mona znale pola indeksu" + Chr( 13 ) + "Nie mona szuka" + Chr( 13 ) , ;
         Chr( 13 ) + "Nie mona szuka wg" + Chr( 13 ) + "pola memo lub logicznego" + Chr( 13 )         , ;
         Chr( 13 ) + "Rekordu nie znaleziono" + Chr( 13 )                                                     , ;
         Chr( 13 ) + "Zbyt wiele kolumn" + Chr( 13 ) + "Raport nie mieci si na arkuszu" + Chr( 13 )      }
      _HMG_aABMLangLabel  := { "Rekord"              , ;
         "Liczba rekordw"        , ;
         "      (Nowy)"        , ;
         "    (Edycja)"        , ;
         "Wprowad numer rekordu" , ;
         "Szukaj"                , ;
         "Szukaj tekstu"         , ;
         "Szukaj daty"         , ;
         "Szukaj liczby"       , ;
         "Definicja Raportu"   , ;
         "Kolumny Raportu"      , ;
         "Dostpne kolumny"     , ;
         "Pierwszy rekord"      , ;
         "Ostatni rekord"        , ;
         "Raport z "          , ;
         "Data:"               , ;
         "Pierwszy rekord:"     , ;
         "Ostatni rekord:"       , ;
         "Sortowanie wg:"         , ;
         "Tak"                 , ;
         "Nie"                  , ;
         "Strona "               , ;
         " z "                 }
      _HMG_aABMLangButton := { "Zamknij"    , ;
         "Nowy"      , ;
         "Edytuj"     , ;
         "Usu"   , ;
         "Znajd"     , ;
         "Id do"     , ;
         "Raport"   , ;
         "Pierwszy"    , ;
         "Poprzedni" , ;
         "Nastpny"     , ;
         "Ostatni"     , ;
         "Zapisz"     , ;
         "Rezygnuj"   , ;
         "Dodaj"      , ;
         "Usu"   , ;
         "Drukuj"    , ;
         "Zamknij"     }
      _HMG_aABMLangError  := { "EDIT, bdna nazwa bazy"                                  , ;
         "EDIT, baza ma wicej ni 16 pl"                   , ;
         "EDIT, tryb odwierzania poza zakresem (zobacz raport bdw)"      , ;
         "EDIT, liczba zdarze podstawowych poza zakresem (zobacz raport bdw)" , ;
         "EDIT, lista zdarze poza zakresem (zobacz raport bdw)"  }

      // EDIT EXTENDED

      _HMG_aLangButton := {          ;
         "&Zamknij",        ; // 1
      "&Nowy",           ; // 2
      "&Modyfikuj",      ; // 3
      "&Kasuj",          ; // 4
      "&Znajd",         ; // 5
      "&Drukuj",         ; // 6
      "&Porzu",         ; // 7
      "&Ok",             ; // 8
      "&Kopiuj",         ; // 9
      "&Aktywuj Filtr",  ; // 10
      "&Deaktywuj Filtr", ; // 11
      "&Przywr",        ; // 12
		"Retry"             } // 13

      _HMG_aLangLabel := {                       ;
         "Brak",                        ; // 1
      "Rekord",                      ; // 2
      "Suma",                        ; // 3
      "Aktywny indeks",              ; // 4
      "Opcje",                       ; // 5
      "Nowy rekord",                 ; // 6
      "Modyfikuj rekord",            ; // 7
      "Wybierz rekord",              ; // 8
      "Znajd rekord",               ; // 9
      "Opcje druku",                 ; // 10
      "Dostpne pola",               ; // 11
      "Pola do druku",               ; // 12
      "Dostpne drukarki",           ; // 13
      "Pierwszy rekord do druku",    ; // 14
      "Ostatni rekord do druku",     ; // 15
      "Skasuj rekord",               ; // 16
      "Podgld",                     ; // 17
      "Poka miniatury",             ; // 18
      "Stan filtru: ",               ; // 19
      "Filtrowane: ",                ; // 20
      "Opcje filtrowania" ,          ; // 21
      "Pola bazy danych" ,           ; // 22
      "Operator porwnania",         ; // 23
      "Warto filtru",              ; // 24
      "Wybierz pola do filtru",      ; // 25
      "Wybierz operator porwnania", ; // 26
      "Rwna si",                   ; // 27
      "Nie rwna si",               ; // 28
      "Wikszy ",                    ; // 29
      "Mniejszy ",                   ; // 30
      "Wikszy lub rwny ",          ; // 31
      "Mniejszy lub rwny"           } // 32
      _HMG_aLangUser := { ;
         CRLF + "Aktywny obszar nie odnaleziony   "  + CRLF + "Wybierz obszar przed wywoaniem EDIT   " + CRLF,   ; // 1
      "Poszukiwany cig znakw (dowolny tekst)",                                                               ; // 2
      "Poszukiwana warto (dowolna liczba)",                                                                  ; // 3
      "Wybierz dat",                                                                                          ; // 4
      "Check for true value",                                                                                  ; // 5
      "Wprowad warto",                                                                                      ; // 6
      "Wybierz dowolny rekord i nacicij OK",                                                                  ; // 7
      CRLF + "Wybrae opcj kasowania rekordu   " + CRLF + "Czy jeste pewien ?    " + CRLF,                  ; // 8
      CRLF + "Brak aktywnych indeksw   " + CRLF + "Wybierz    " + CRLF,                                       ; // 9
      CRLF + "Nie mona szuka w polach typu MEMO lub LOGIC   " + CRLF,                                        ; // 10
      CRLF + "Rekord nie znaleziony   " + CRLF,                                                                ; // 11
      "Wybierz rekord ktry naley doda do listy",                                                            ; // 12
      "Wybierz rekord ktry naley wyczy z listy",                                                          ; // 13
      "Wybierz drukark",                                                                                      ; // 14
      "Kliknij na przycisk by doda pole",                                                                     ; // 15
      "Kliknij na przycisk by odj pole",                                                                     ; // 16
      "Kliknij, aby wybra pierwszy rekord do druku",                                                          ; // 17
      "Kliknij, aby wybra ostatni rekord do druku",                                                           ; // 18
      CRLF + "Brak pl do wczenia   " + CRLF,                                                                ; // 19
      CRLF + "Najpierw wybierz pola do wczenia   " + CRLF,                                                   ; // 20
      CRLF + "Brak pl do wyczenia   " + CRLF,                                                               ; // 21
      CRLF + "Najpierw wybierz pola do wyczenia   " + CRLF,                                                  ; // 22
      CRLF + "Nie wybrae adnych pl   " + CRLF + "Najpierw wybierz pola do wczenia do wydruku   " + CRLF, ; // 23
      CRLF + "Za wiele pl   " + CRLF + "Zredukuj liczb pl   " + CRLF,                                       ; // 24
      CRLF + "Drukarka nie gotowa   " + CRLF,                                                                  ; // 25
      "Porzdek wg",                                                                                           ; // 26
      "Od rekordu",                                                                                            ; // 27
      "Do rekordu",                                                                                            ; // 28
      "Tak",                                                                                                   ; // 29
      "Nie",                                                                                                   ; // 30
      "Strona:",                                                                                               ; // 31
      CRLF + "Wybierz drukark   " + CRLF,                                                                     ; // 32
      "Filtrowanie wg",                                                                                        ; // 33
      CRLF + "Brak aktywnego filtru    " + CRLF,                                                               ; // 34
      CRLF + "Nie mona filtrowa wg. pl typu MEMO    " + CRLF,                                               ; // 35
      CRLF + "Wybierz pola dla filtru    " + CRLF,                                                             ; // 36
      CRLF + "Wybierz operator porwnania dla filtru    " + CRLF,                                              ; // 37
      CRLF + "Wpisz dowoln warto dla filtru    " + CRLF,                                                    ; // 38
      CRLF + "Brak aktywnego filtru    " + CRLF,                                                               ; // 39
      CRLF + "Deaktywowa filtr?   " + CRLF,                                                                   ; // 40
      CRLF + "Rekord zablokowany przez innego uytkownika" + CRLF,                                             ; // 41
      CRLF + "Czy przwrci skasowny   " + CRLF + "Czy jeste pewien?    " + CRLF                              } // 42

   CASE cLang == "PT"  // Portuguese
      /////////////////////////////////////////////////////////////
      // PORTUGUESE
      ////////////////////////////////////////////////////////////

      // MISC MESSAGES

      _HMG_MESSAGE [1] := "Voc tem Certeza ?"
      _HMG_MESSAGE [2] := "Fechar Janela"
      _HMG_MESSAGE [3] := "Fechamento no permitido"
      _HMG_MESSAGE [4] := "Programa j est em execuo"
      _HMG_MESSAGE [5] := "Edita"
      _HMG_MESSAGE [6] := "Ok"
      _HMG_MESSAGE [7] := "Cancela"
      _HMG_MESSAGE [8] := "Aplicar"
      _HMG_MESSAGE [9] := "Pg."
      _HMG_MESSAGE [10] := 'Ateno'
      _HMG_MESSAGE [11] := 'Informao'
      _HMG_MESSAGE [12] := 'Pare'

      // BROWSE

      _HMG_BRWLangButton  := { "Incluir"  , ;
         "Alterar"  , ;
         "Cancelar" , ;
         "OK"        }
      _HMG_BRWLangError   := { "Window: "                                         , ;
         " Erro no definido. Programa ser fechado"        , ;
         "Erro na MiniGUI.lib"                              , ;
         "Controle: "                                       , ;
         "Desligado "                                       , ;
         "No pronto. Programa ser fechado"                , ;
         "Browse: Tipo Invlido !!!. Programa ser fechado" , ;
         "Browse: Edio no pode ser efetivada, campo no pertence a essa rea. Programa ser fechado" , ;
         "Arquivo em uso no pode ser editado !!!"          , ;
         "Aguarde..."                                       , ;
         "Dado Invlido"                                     }
      _HMG_BRWLangMessage := { "Voc tem Certeza ?" , "Apaga Registro" }

      // EDIT

      _HMG_aABMLangUser   := { Chr( 13 ) + "Ser apagado o registro atual" + Chr( 13 ) + "Tem certeza ?"                          + Chr( 13 ) , ;
         Chr( 13 ) + "No existe um ndice ativo"    + Chr( 13 ) + "No  possvel realizar a busca"        + Chr( 13 ) , ;
         Chr( 13 ) + "No encontrado o campo ndice" + Chr( 13 ) + "No  possvel realizar a busca"        + Chr( 13 ) , ;
         Chr( 13 ) + "No  possvel realizar busca" + Chr( 13 ) + "por campos memo ou lgicos"             + Chr( 13 ) , ;
         Chr( 13 ) + "Registro no encontrado"       + Chr( 13 )                                                      , ;
         Chr( 13 ) + "Includas colunas em excesso"  + Chr( 13 ) + "A listagem completa no caber na tela" + Chr( 13 )  }
      _HMG_aABMLangLabel  := { "Registro Atual"                 , ;
         "Total Registros"                , ;
         "(Novo)"                         , ;
         "(Editar)"                       , ;
         "Introduza o nmero do registro" , ;
         "Buscar"                         , ;
         "Texto a buscar"                 , ;
         "Data a buscar"                  , ;
         "Nmero a buscar"                , ;
         "Definio da lista"             , ;
         "Colunas da lista"               , ;
         "Colunas disponveis"            , ;
         "Registro inicial"               , ;
         "Registro final"                 , ;
         "Lista de "                      , ;
         "Data:"                          , ;
         "Primeiro registro:"             , ;
         "ltimo registro:"               , ;
         "Ordenado por:"                  , ;
         "Sim"                            , ;
         "No"                            , ;
         "Pgina "                        , ;
         " de "                            }
      _HMG_aABMLangButton := { "Fechar"           , ;
         "Novo"             , ;
         "Modificar"        , ;
         "Eliminar"         , ;
         "Buscar"           , ;
         "Ir ao registro"   , ;
         "Listar"           , ;
         "Primeiro"         , ;
         "Anterior"         , ;
         "Seguinte"         , ;
         "ltimo"           , ;
         "Guardar"          , ;
         "Cancelar"         , ;
         "Juntar"           , ;
         "Sair"             , ;
         "Imprimir"         , ;
         "Fechar"            }
      _HMG_aABMLangError  := { "EDIT, no foi especificada a rea"                                      , ;
         "EDIT, A rea contm mais de 16 campos"                                  , ;
         "EDIT, Atualizao fora do limite (por favor, comunique o erro)"         , ;
         "EDIT, Evento principal fora do limite (por favor, comunique o erro)"    , ;
         "EDIT, Evento mostrado est fora do limite (por favor, comunique o erro)" }

      // EDIT EXTENDED

      _HMG_aLangButton    := { "&Sair",             ; // 1
      "&Novo",             ; // 2
      "&Alterar",          ; // 3
      "&Eliminar",         ; // 4
      "&Localizar",        ; // 5
      "&Imprimir",         ; // 6
      "&Cancelar",         ; // 7
      "&Aceitar",          ; // 8
      "&Copiar",           ; // 9
      "&Ativar Filtro",    ; // 10
      "&Desativar Filtro", ; // 11
      "&Restaurar",        ; // 12
		"Retry"              } // 13

      _HMG_aLangLabel     := { ;
      "Nenhum",                             ; // 1
      "Registro",                           ; // 2
      "Total",                              ; // 3
      "ndice ativo",                       ; // 4
      "Opo",                              ; // 5
      "Novo registro",                      ; // 6
      "Modificar registro",                 ; // 7
      "Selecionar registro",                ; // 8
      "Localizar registro",                 ; // 9
      "Opo de impresso",                 ; // 10
      "Campos disponveis",                 ; // 11
      "Campos selecionados",                ; // 12
      "Impressoras disponveis",            ; // 13
      "Primeiro registro a imprimir",       ; // 14
      "ltimo registro a imprimir",         ; // 15
      "Apagar registro",                    ; // 16
      "Visualizar impresso",               ; // 17
      "Pginas em miniatura",               ; // 18
      "Condio do filtro: ",               ; // 19
      "Filtrado: ",                         ; // 20
      "Opes do filtro" ,                  ; // 21
      "Campos da tabela" ,                  ; // 22
      "Operador de comparao",             ; // 23
      "Valor de comparao",                ; // 24
      "Selecione o campo a filtrar",        ; // 25
      "Selecione o operador de comparao", ; // 26
      "Igual",                              ; // 27
      "Diferente",                          ; // 28
      "Maior que",                          ; // 29
      "Menor que",                          ; // 30
      "Maior ou igual que",                 ; // 31
      "Menor ou igual que"                  } // 32
      _HMG_aLangUser      := {  CRLF + "No h uma rea ativa   "  + CRLF + "Por favor, selecione uma rea antes de chamar a EDIT EXTENDED   " + CRLF, ; // 1
      "Introduza o valor do campo (texto)",                                                                                  ; // 2
      "Introduza o valor do campo (numrico)",                                                                               ; // 3
      "Selecione a data",                                                                                                    ; // 4
      "Ative o indicador para valor verdadeiro",                                                                             ; // 5
      "Introduza o valor do campo",                                                                                          ; // 6
      "Selecione um registro e tecle Ok",                                                                                    ; // 7
      CRLF + "Confirma apagar o registro ativo   " + CRLF + "Tem certeza?     " + CRLF,                                      ; // 8
      CRLF + "No h um ndice selecionado    " + CRLF + "Por favor, selecione um   " + CRLF,                                ; // 9
      CRLF + "No se pode realizar busca por campos tipo memo ou lgico   " + CRLF,                                          ; // 10
      CRLF + "Registro no encontrado   " + CRLF,                                                                            ; // 11
      "Selecione o campo a incluir na lista",                                                                                ; // 12
      "Selecione o campo a excluir da lista",                                                                                ; // 13
      "Selecione a impressora",                                                                                              ; // 14
      "Pressione o boto para incluir o campo",                                                                              ; // 15
      "Pressione o boto para excluir o campo",                                                                              ; // 16
      "Pressione o boto para selecionar o primeiro registro a imprimir",                                                    ; // 17
      "Pressione o boto para selecionar o ltimo registro a imprimir",                                                      ; // 18
      CRLF + "Foram includos todos os campos   " + CRLF,                                                                    ; // 19
      CRLF + "Primeiro seleccione o campo a incluir   " + CRLF,                                                              ; // 20
      CRLF + "No h campos para excluir   " + CRLF,                                                                         ; // 21
      CRLF + "Primeiro selecione o campo a excluir   " + CRLF,                                                               ; // 22
      CRLF + "No foi selecionado nenhum campo   " + CRLF,                                                                   ; // 23
      CRLF + "A lista no cabe na pgina   " + CRLF + "Reduza o nmero de campos   " + CRLF,                                 ; // 24
      CRLF + "A impressora no est disponvel   " + CRLF,                                                                   ; // 25
      "Ordenado por",                                                                                                        ; // 26
      "Do registro",                                                                                                         ; // 27
      "At registro",                                                                                                        ; // 28
      "Sim",                                                                                                                 ; // 29
      "No",                                                                                                                 ; // 30
      "Pgina:",                                                                                                             ; // 31
      CRLF + "Por favor, selecione uma impressora   " + CRLF,                                                                ; // 32
      "Filtrado por",                                                                                                        ; // 33
      CRLF + "No h um filtro ativo    " + CRLF,                                                                            ; // 34
      CRLF + "No se pode filtrar por campos memo    " + CRLF,                                                               ; // 35
      CRLF + "Selecione o campo a filtrar    " + CRLF,                                                                       ; // 36
      CRLF + "Selecione o operador de comparao    " + CRLF,                                                                ; // 37
      CRLF + "Introduza o valor do filtro    " + CRLF,                                                                       ; // 38
      CRLF + "No h nenhum filtro ativo    " + CRLF,                                                                        ; // 39
      CRLF + "Eliminar o filtro ativo ?   " + CRLF,                                                                          ; // 40
      CRLF + "Registro bloqueado por outro usurio    " + CRLF,                                                              ; // 41
      CRLF + "Voc vai restabelecer o registro apagado   " + CRLF + "Tem certeza ?    " + CRLF                               } // 42

   CASE cLang == "RU"  // Russian
      /////////////////////////////////////////////////////////////
      // RUSSIAN
      ////////////////////////////////////////////////////////////

      // MISC MESSAGES

      _HMG_MESSAGE [1] := '  ?'
      _HMG_MESSAGE [2] := ' '
      _HMG_MESSAGE [3] := '  '
      _HMG_MESSAGE [4] := '  '
      _HMG_MESSAGE [5] := ''
      _HMG_MESSAGE [6] := ''
      _HMG_MESSAGE [7] := ''
      _HMG_MESSAGE [8] := ''
      _HMG_MESSAGE [9] := '.'
      _HMG_MESSAGE[10] := ''
      _HMG_MESSAGE[11] := ''
      _HMG_MESSAGE[12] := ''

      // BROWSE

      _HMG_BRWLangButton := { ;
         "" , ;
         "" , ;
         ""   , ;
         "OK" }
      _HMG_BRWLangError  := { ;
         ": "                                                  , ;
         "  .  "                      , ;
         "MiniGUI "                                          , ;
         " : "                                    , ;
         "  "                                                    , ;
         "  .  "                      , ;
         "Browse:    .  " , ;
         "Browse: Append          .  ", ;
         "    "        , ;
         ""                                          , ;
         "  " }
      _HMG_BRWLangMessage := { '  ?' , ' ' }

      // EDIT

      _HMG_aABMLangUser := { ;
         Chr( 13 ) + " ." + Chr( 13 ) + "  ?" + Chr( 13 )                       , ;
         Chr( 13 ) + "  " + Chr( 13 ) + " " + Chr( 13 )         , ;
         Chr( 13 ) + "  " + Chr( 13 ) + " " + Chr( 13 )         , ;
         Chr( 13 ) + "  " + Chr( 13 ) + "   " + Chr( 13 ) , ;
         Chr( 13 ) + "  " + Chr( 13 )                                                   , ;
         Chr( 13 ) + "  " + Chr( 13 ) + "    " + Chr( 13 ) }
      _HMG_aABMLangLabel := { ;
         ""              , ;
         " "       , ;
         "     ()"        , ;
         "  ()"        , ;
         "  ", ;
         ""               , ;
         " "         , ;
         " "          , ;
         " "         , ;
         " "    , ;
         " "      , ;
         " "   , ;
         " "    , ;
         " "     , ;
         "  "          , ;
         ":"               , ;
         " :"      , ;
         " :"    , ;
         " :"     , ;
         ""                  , ;
         ""                 , ;
         " "           , ;
         "  "                 }
      _HMG_aABMLangButton := { ;
         ""   , ;
         ""     , ;
         ""  , ;
         ""   , ;
         ""     , ;
         ""   , ;
         ""     , ;
         ""    , ;
         ""     , ;
         ""    , ;
         "" , ;
         "" , ;
         ""    , ;
         ""  , ;
         ""   , ;
         ""    , ;
         ""    }
      _HMG_aABMLangError := { ;
         "EDIT,     "                     , ;
         "EDIT,    16 "                      , ;
         "EDIT,     (  )", ;
         "EDIT,     (  )"   , ;
         "EDIT,      (  )" }

      // EDIT EXTENDED

      _HMG_aLangButton := { ;
         "&",           ; // 1
         "&",           ; // 2
         "&",            ; // 3
         "&",           ; // 4
         "&",             ; // 5
         "&",            ; // 6
         "&",            ; // 7
         "&",                ; // 8
         "&",             ; // 9
         "&. ",       ; // 10
         "& ",      ; // 11
         "&",      ; // 12
			"&"       } // 13

      _HMG_aLangLabel := { ;
         "",                          ; // 1
         "",                       ; // 2
         "",                        ; // 3
         "",                 ; // 4
         "",                    ; // 5
         " ",                 ; // 6
         " ",              ; // 7
         " ",               ; // 8
         " ",                 ; // 9
         " ",             ; // 10
         " ",               ; // 11
         "  ",              ; // 12
         " ",           ; // 13
         "   ",       ; // 14
         "  ",     ; // 15
         " ",               ; // 16
         "",                     ; // 17
         " ",            ; // 18
         " : ",            ; // 19
         ": ",                     ; // 20
         " " ,           ; // 21
         "  " ,            ; // 22
         " ",          ; // 23
         " ",             ; // 24
         "   ",       ; // 25
         "  ",    ; // 26
         "",                        ; // 27
         " ",                     ; // 28
         "",                       ; // 29
         "",                       ; // 30
         "  ",             ; // 31
         "  "           }    // 32
      _HMG_aLangUser := { ;
         CRLF + "   ."  + CRLF + "      EDIT" + CRLF, ; // 1
         "  ",                                                                               ; // 2
         " ",                                                                                            ; // 3
         " ",                                                                                             ; // 4
         " ",                                                                                      ; // 5
         "  ",                                                                                    ; // 6
         "     OK",                                                                       ; // 7
         CRLF + "    " + CRLF + " ?    " + CRLF,                                  ; // 8
         CRLF + "  " + CRLF + "    " + CRLF,                                ; // 9
         CRLF + "         " + CRLF,                                ; // 10
         CRLF + "    " + CRLF,                                                                        ; // 11
         "     ",                                                                       ; // 12
         "   ",                                                                                  ; // 13
         " ",                                                                                           ; // 14
         "      ",                                                                ; // 15
         "      ",                                                             ; // 16
         ",    ",                                                                      ; // 17
         ",    ",                                                                    ; // 18
         CRLF + "   " + CRLF,                                                                      ; // 19
         CRLF + "    " + CRLF,                                                                  ; // 20
         CRLF + "   " + CRLF,                                                                     ; // 21
         CRLF + "    " + CRLF,                                                                 ; // 22
         CRLF + "   " + CRLF + "    " + CRLF,                             ; // 23
         CRLF + "   " + CRLF + "   " + CRLF,                                   ; // 24
         CRLF + "    " + CRLF,                                                                         ; // 25
         " ",                                                                                            ; // 26
         "  ",                                                                                               ; // 27
         "  ",                                                                                               ; // 28
         "",                                                                                                       ; // 29
         "",                                                                                                      ; // 30
         ":",                                                                                                ; // 31
         CRLF + "   " + CRLF,                                                                         ; // 32
         " ",                                                                                         ; // 33
         CRLF + "       " + CRLF,                                                                 ; // 34
         CRLF + "     " + CRLF,                                                           ; // 35
         CRLF + "       " + CRLF,                                                              ; // 36
         CRLF + "    " + CRLF,                                                        ; // 37
         CRLF + "    " + CRLF,                                                        ; // 38
         CRLF + "     " + CRLF,                                                                   ; // 39
         CRLF + "  ?   " + CRLF,                                                                          ; // 40
         CRLF + "    " + CRLF,                                                   ; // 41
         CRLF + "    " + CRLF + " ?    " + CRLF                             } // 42

   CASE cLang == "UK" .OR. cLang == "UA"  // Ukrainian
      /////////////////////////////////////////////////////////////
      // UKRAINIAN
      ////////////////////////////////////////////////////////////

      // MISC MESSAGES

      _HMG_MESSAGE[ 1 ] := ' i ?'
      _HMG_MESSAGE[ 2 ] := ' i.'
      _HMG_MESSAGE[ 3 ] := '  .'
      _HMG_MESSAGE[ 4 ] := ( ' .' + CRLF + '  i i .' )
      _HMG_MESSAGE[ 5 ] := 'i'
      _HMG_MESSAGE[ 6 ] := ''
      _HMG_MESSAGE[ 7 ] := ''
      _HMG_MESSAGE[ 8 ] := ''
      _HMG_MESSAGE[ 9 ] := '.'
      _HMG_MESSAGE[10 ] := '!'
      _HMG_MESSAGE[11 ] := 'I'
      _HMG_MESSAGE[12 ] := ''

      // BROWSE

      _HMG_BRWLangButton := { ""    , ;
         "i"   , ;
         "" , ;
         ""    }
      _HMG_BRWLangError  := { "i: "                                              , ;
         "  .  "                        , ;
         "MiniGUI "                                         , ;
         " i: "                                    , ;
         "  "                                                     , ;
         "  .  "                      , ;
         "Browse:    i.  "   , ;
         "Browse: Append      i  i.  ", ;
         "   i "              , ;
         ""                                            , ;
         " i i"                                 }
      _HMG_BRWLangMessage := { ' i ?' , ' ' }

      // EDIT

      _HMG_aABMLangUser   := { Chr( 13 ) + " ." + Chr( 13 ) + " i ?" + Chr( 13 )             , ;
         Chr( 13 ) + "ii i " + Chr( 13 ) + " " + Chr( 13 )       , ;
         Chr( 13 ) + "i i " + Chr( 13 ) + " " + Chr( 13 )         , ;
         Chr( 13 ) + "  " + Chr( 13 ) + "i  i " + Chr( 13 ) , ;
         Chr( 13 ) + "  " + Chr( 13 )                                         , ;
         Chr( 13 ) + "  " + Chr( 13 ) + "i  i  i" + Chr( 13 ) }
      _HMG_aABMLangLabel  := { ""           , ;
         " i"      , ;
         "     ()"         , ;
         "  (i)"         , ;
         "i  ", ;
         ""               , ;
         " "        , ;
         " "         , ;
         " "        , ;
         " i"  , ;
         " i"       , ;
         "i "    , ;
         " "    , ;
         " "      , ;
         "i  "           , ;
         ":"               , ;
         " :"       , ;
         " :"     , ;
         " :"      , ;
         ""                 , ;
         "i"                  , ;
         "i "           , ;
         "  "                  }
      _HMG_aABMLangButton := { "", ;
         ""       , ;
         "i"    , ;
         ""   , ;
         ""      , ;
         ""    , ;
         "i"       , ;
         ""      , ;
         ""      , ;
         ""     , ;
         ""    , ;
         ""   , ;
         "i"    , ;
         ""     , ;
         ""   , ;
         ""       , ;
         ""     }
      _HMG_aABMLangError  := { "EDIT,   i'  i"                    , ;
         "EDIT,   i 16 i"                        , ;
         "EDIT,    i (i  )", ;
         "EDIT,  i  i (i  )"    , ;
         "EDIT,  i i  i (i  )" }

      // EDIT EXTENDED

      _HMG_aLangButton := { ;
         "&",           ; // 1
      "&",          ; // 2
      "&",            ; // 3
      "&",          ; // 4
      "&",            ; // 5
      "&",              ; // 6
      "&",         ; // 7
      "&",            ; // 8
      "&i",             ; // 9
      ". &i",       ; // 10
      "& i",      ; // 11
      "³&",		    ; // 12
		"&"        } // 13

      _HMG_aLangLabel := { ;
      "i",                              ; // 1
      "",                           ; // 2
      "",                          ; // 3
      "",                   ; // 4
      "",                       ; // 5
      " ",                     ; // 6
      "i ",                   ; // 7
      " ",                   ; // 8
      " ",                    ; // 9
      " ",                 ; // 10
      "i ",                   ; // 11
      "  ",                  ; // 12
      "i ",               ; // 13
      "   ",         ; // 14
      "  ",          ; // 15
      " ",                  ; // 16
      "",                        ; // 17
      "i ii",               ; // 18
      " i: ",                 ; // 19
      "i: ",                        ; // 20
      " i" ,              ; // 21
      "  " ,                ; // 22
      " i",         ; // 23
      " i",                ; // 24
      "i   ",          ; // 25
      "i  i",   ; // 26
      "i",                        ; // 27
      " i",                     ; // 28
      "i",                          ; // 29
      "",                           ; // 30
      "i  i",             ; // 31
      "  i"           }     // 32
      _HMG_aLangUser := { ;
         CRLF + "   i."  + CRLF + "i -     EDIT" + CRLF, ; // 1
      "i  ",                                                                                 ; // 2
      "i ",                                                                                             ; // 3
      "i ",                                                                                              ; // 4
      "i ",                                                                                          ; // 5
      "i  ",                                                                                     ; // 6
      "i -  i i OK",                                                                    ; // 7
      CRLF + "    " + CRLF + " ?    " + CRLF,                                   ; // 8
      CRLF + "i  " + CRLF + "i   i " + CRLF,                                  ; // 9
      CRLF + "   i i i    " + CRLF,                                     ; // 10
      CRLF + "    " + CRLF,                                                                         ; // 11
      ", i  ",                                                                                  ; // 12
      " i  ",                                                                                    ; // 13
      "i ",                                                                                            ; // 14
      "i      ",                                                             ; // 15
      "i      ",                                                               ; // 16
      ",    ",                                                                        ; // 17
      ",   ",                                                                             ; // 18
      CRLF + "ii i  " + CRLF,                                                                     ; // 19
      CRLF + "    " + CRLF,                                                                    ; // 20
      CRLF + "ii    " + CRLF,                                                               ; // 21
      CRLF + "    " + CRLF,                                                                   ; // 22
      CRLF + "ii i  " + CRLF + "    " + CRLF,                               ; // 23
      CRLF + "  i " + CRLF + "i  ii " + CRLF,                                     ; // 24
      CRLF + "  i " + CRLF,                                                                     ; // 25
      " ",                                                                                            ; // 26
      "i  ",                                                                                               ; // 27
      "  ",                                                                                                ; // 28
      "",                                                                                                       ; // 29
      "i",                                                                                                        ; // 30
      "i:",                                                                                                 ; // 31
      CRLF + "i   " + CRLF,                                                                          ; // 32
      "ii ",                                                                                         ; // 33
      CRLF + "   i   " + CRLF,                                                                    ; // 34
      CRLF + " i  i  " + CRLF,                                                             ; // 35
      CRLF + "i   i    " + CRLF,                                                                ; // 36
      CRLF + "i   i" + CRLF,                                                                ; // 37
      CRLF + "i   i" + CRLF,                                                                ; // 38
      CRLF + "ii i i   " + CRLF,                                                                 ; // 39
      CRLF + " i ?   " + CRLF,                                                                           ; // 40
      CRLF + "  i  " + CRLF,                                                       ; // 41
      CRLF + "    " + CRLF + " ?    " + CRLF                                  } // 42

   CASE cLang == "ES"  // Spanish
      /////////////////////////////////////////////////////////////
      // SPANISH
      ////////////////////////////////////////////////////////////

      // MISC MESSAGES

      _HMG_MESSAGE [1] := 'Est seguro ?'
      _HMG_MESSAGE [2] := 'Cerrar Ventana'
      _HMG_MESSAGE [3] := 'Operacin no permitida'
      _HMG_MESSAGE [4] := 'EL programa ya est ejecutndose'
      _HMG_MESSAGE [5] := 'Editar'
      _HMG_MESSAGE [6] := 'Aceptar'
      _HMG_MESSAGE [7] := 'Cancelar'
      _HMG_MESSAGE [8] := 'Apply'
      _HMG_MESSAGE [9] := 'Pag.'
      _HMG_MESSAGE [10] := 'Atencion'
      _HMG_MESSAGE [11] := 'Informacin'
      _HMG_MESSAGE [12] := 'Detener'

      // BROWSE

      _HMG_BRWLangButton := { "Agregar"    , ;
         "Editar"     , ;
         "Cancelar"   , ;
         "Aceptar"     }
      _HMG_BRWLangError  := { "Window: "                                              , ;
         " no est definida. Ejecucin terminada"                , ;
         "MiniGUI Error"                                         , ;
         "Control: "                                             , ;
         " De "                                                  , ;
         " ya definido. Ejecucin terminada"                     , ;
         "Browse: Tipo no permitido. Ejecucin terminada"        , ;
         "Browse: La calusula APPEND no puede ser usada con campos no pertenecientes al area del BROWSE. Ejecucin terminada", ;
         "El registro est siendo editado por otro usuario"      , ;
         "Peligro"                                               , ;
         "Entrada no vlida"                                      }
      _HMG_BRWLangMessage := { 'Est Seguro ?' , 'Eliminar Registro' }

      // EDIT

      _HMG_aABMLangUser   := { Chr( 13 ) + "Va a eliminar el registro actual" + Chr( 13 ) + " Est seguro ?" + Chr( 13 )                 , ;
         Chr( 13 ) + "No hay un indice activo" + Chr( 13 ) + "No se puede realizar la busqueda" + Chr( 13 )         , ;
         Chr( 13 ) + "No se encuentra el campo indice" + Chr( 13 ) + "No se puede realizar la busqueda" + Chr( 13 ) , ;
         Chr( 13 ) + "No se pueden realizar busquedas" + Chr( 13 ) + "por campos memo o lgico" + Chr( 13 )         , ;
         Chr( 13 ) + "Registro no encontrado" + Chr( 13 )                                                     , ;
         Chr( 13 ) + "Ha inclido demasiadas columnas" + Chr( 13 ) + "El listado no cabe en la hoja" + Chr( 13 )      }
      _HMG_aABMLangLabel  := { "Registro Actual"                  , ;
         "Registros Totales"                , ;
         "     (Nuevo)"                     , ;
         "    (Editar)"                     , ;
         "Introducca el nmero de registro" , ;
         "Buscar"                           , ;
         "Texto a buscar"                   , ;
         "Fecha a buscar"                   , ;
         "Nmero a buscar"                  , ;
         "Definicin del listado"           , ;
         "Columnas del listado"             , ;
         "Columnas disponibles"             , ;
         "Registro inicial"                 , ;
         "Registro final"                   , ;
         "Listado de "                      , ;
         "Fecha:"                           , ;
         "Primer registro:"                 , ;
         "Ultimo registro:"                 , ;
         "Ordenado por:"                    , ;
         "Si"                               , ;
         "No"                               , ;
         "Pagina "                          , ;
         " de "                              }
      _HMG_aABMLangButton := { "Cerrar"           , ;
         "Nuevo"            , ;
         "Modificar"        , ;
         "Eliminar"         , ;
         "Buscar"           , ;
         "Ir al registro"   , ;
         "Listado"          , ;
         "Primero"          , ;
         "Anterior"         , ;
         "Siguiente"        , ;
         "Ultimo"           , ;
         "Guardar"          , ;
         "Cancelar"         , ;
         "Aadir"           , ;
         "Quitar"           , ;
         "Imprimir"         , ;
         "Cerrar"            }
      _HMG_aABMLangError  := { "EDIT, No se ha especificado el area"                                  , ;
         "EDIT, El area contiene ms de 16 campos"                              , ;
         "EDIT, Refesco fuera de rango (por favor comunique el error)"          , ;
         "EDIT, Evento principal fuera de rango (por favor comunique el error)" , ;
         "EDIT, Evento listado fuera de rango (por favor comunique el error)"    }

      // EDIT EXTENDED

      _HMG_aLangButton := {            ;
         "&Cerrar",           ; // 1
      "&Nuevo",            ; // 2
      "&Modificar",        ; // 3
      "&Eliminar",         ; // 4
      "&Buscar",           ; // 5
      "&Imprimir",         ; // 6
      "&Cancelar",         ; // 7
      "&Aceptar",          ; // 8
      "&Copiar",           ; // 9
      "&Activar Filtro",   ; // 10
      "&Desactivar Filtro", ; // 11
      "&Restaurar",         ; // 12
		"Retry"               } // 13

      _HMG_aLangLabel := {                                 ;
         "Ninguno",                               ; // 1
      "Registro",                              ; // 2
      "Total",                                 ; // 3
      "Indice activo",                         ; // 4
      "Opciones",                              ; // 5
      "Nuevo registro",                        ; // 6
      "Modificar registro",                    ; // 7
      "Seleccionar registro",                  ; // 8
      "Buscar registro",                       ; // 9
      "Opciones de impresin",                 ; // 10
      "Campos disponibles",                    ; // 11
      "Campos del listado",                    ; // 12
      "Impresoras disponibles",                ; // 13
      "Primer registro a imprimir",            ; // 14
      "Ultimo registro a imprimir",            ; // 15
      "Borrar registro",                       ; // 16
      "Vista previa",                          ; // 17
      "Pginas en miniatura",                  ; // 18
      "Condicin del filtro: ",                ; // 19
      "Filtrado: ",                            ; // 20
      "Opciones de filtrado" ,                 ; // 21
      "Campos de la bdd" ,                     ; // 22
      "Operador de comparacin",               ; // 23
      "Valor de comparacin",                  ; // 24
      "Seleccione el campo a filtrar",         ; // 25
      "Seleccione el operador de comparacin", ; // 26
      "Igual",                                 ; // 27
      "Distinto",                              ; // 28
      "Mayor que",                             ; // 29
      "Menor que",                             ; // 30
      "Mayor o igual que",                     ; // 31
      "Menor o igual que"                      } // 32
      _HMG_aLangUser := { ;
         CRLF + "No hay un area activa   "  + CRLF + "Por favor seleccione un area antes de llamar a EDIT EXTENDED   " + CRLF,       ; // 1
      "Introduzca el valor del campo (texto)",                                                                                      ; // 2
      "Introduzca el valor del campo (numrico)",                                                                                    ; // 3
      "Seleccione la fecha",                                                                                                      ; // 4
      "Active la casilla para indicar un valor verdadero",                                                                                                 ; // 5
      "Introduzca el valor del campo",                                                                                                ; // 6
      "Seleccione un registro y pulse aceptar",                                                                                       ; // 7
      CRLF + "Se dispone a borrar el registro activo   " + CRLF + "Esta seguro?    " + CRLF,                  ; // 8
      CRLF + "No se ha seleccionado un indice   " + CRLF + "Por favor seleccione uno   " + CRLF,                            ; // 9
      CRLF + "No se pueden realizar busquedad por campos tipo memo o lgico   " + CRLF,                                                   ; // 10
      CRLF + "Registro no encontrado   " + CRLF,                                                                            ; // 11
      "Seleccione el campo a incluir en el listado",                                                                                  ; // 12
      "Seleccione el campo a excluir del listado",                                                                                ; // 13
      "Seleccione la impresora",                                                                                                   ; // 14
      "Pulse el botn para incluir el campo",                                                                                         ; // 15
      "Pulse el botn para excluir el campo",                                                                                         ; // 16
      "Pulse el botn para seleccionar el primer registro a imprimir",                                                                      ; // 17
      "Pulse el botn para seleccionar el ltimo registro a imprimir",                                                                       ; // 18
      CRLF + "Ha incluido todos los campos   " + CRLF,                                                                   ; // 19
      CRLF + "Primero seleccione el campo a incluir   " + CRLF,                                                           ; // 20
      CRLF + "No hay campos para excluir   " + CRLF,                                                                    ; // 21
      CRLF + "Primero seleccione el campo a excluir   " + CRLF,                                                            ; // 22
      CRLF + "No ha seleccionado ningn campo   " + CRLF,                                              ; // 23
      CRLF + "El listado no cabe en la pgina   " + CRLF + "Reduzca el numero de campos   " + CRLF,                                   ; // 24
      CRLF + "La impresora no est disponible   " + CRLF,                                                                           ; // 25
      "Ordenado por",                                                                                                           ; // 26
      "Del registro",                                                                                                          ; // 27
      "Al registro",                                                                                                            ; // 28
      "Si",                                                                                                                  ; // 29
      "No",                                                                                                                   ; // 30
      "Pgina:",                                                                                                                ; // 31
      CRLF + "Por favor seleccione una impresora   " + CRLF,                                                                     ; // 32
      "Filtrado por",                                                                                                          ; // 33
      CRLF + "No hay un filtro activo    " + CRLF,                                                                  ; // 34
      CRLF + "No se puede filtrar por campos memo    " + CRLF,                                                                ; // 35
      CRLF + "Seleccione el campo a filtrar    " + CRLF,                                                                 ; // 36
      CRLF + "Seleccione el operador de comparacin    " + CRLF,                                                              ; // 37
      CRLF + "Introduzca el valor del filtro    " + CRLF,                                                                 ; // 38
      CRLF + "No hay ningn filtro activo    " + CRLF,                                                              ; // 39
      CRLF + "Eliminar el filtro activo?   " + CRLF,                                                                           ; // 40
      CRLF + "Registro bloqueado por otro usuario    " + CRLF,                                                                   ; // 41
      CRLF + "Se dispone a restaurar el registro suprimido   " + CRLF + "Esta seguro?    " + CRLF,                  } // 42

   CASE cLang == "FI"  // Finnish
      ///////////////////////////////////////////////////////////////////////
      // FINNISH
      ///////////////////////////////////////////////////////////////////////

      // MISC MESSAGES

      _HMG_MESSAGE [1] := 'Oletko varma ?'
      _HMG_MESSAGE [2] := 'Sulje ikkuna'
      _HMG_MESSAGE [3] := 'Sulkeminen ei sallittu'
      _HMG_MESSAGE [4] := 'Ohjelma on jo kynniss'
      _HMG_MESSAGE [5] := 'Korjaa'
      _HMG_MESSAGE [6] := 'Ok'
      _HMG_MESSAGE [7] := 'Keskeyt'
      _HMG_MESSAGE [8] := 'Apply'
      _HMG_MESSAGE [9] := 'Sivu.'
      _HMG_MESSAGE [10] := 'Attention'
      _HMG_MESSAGE [11] := 'Information'
      _HMG_MESSAGE [12] := 'Stop'

      // BROWSE

      _HMG_BRWLangButton := { "Lis"  , ;
         "Korjaa" , ;
         " Keskeyt" , ;
         " OK" }

      _HMG_BRWLangError  := { "Ikkuna: " , ;
         " mrittelemtn. Ohjelma lopetettu" , ;
         "MiniGUI Virhe", ;
         "Kontrolli: ", ;
         " / " , ;
         " On jo mritelty. Ohjelma lopetettu" , ;
         "Browse: Virheellinen tyyppi. Ohjelma lopetettu" , ;
         "Browse: Et voi list kentti jotka eivt ole BROWSEN mrityksess. Ohjelma lopetettu", ;
         "Toinen kyttj korjaa juuri tietuetta" , ;
         "Varoitus" , ;
         "Virheellinen arvo" }

      _HMG_BRWLangMessage := { 'Oletko varma ?' , 'Poista tietue' }

      // EDIT
      _HMG_aABMLangUser   := { Chr( 13 ) + "Poista tietue" + Chr( 13 ) + "Oletko varma?" + Chr( 13 )                  , ;
         Chr( 13 ) + "Indeksi tiedosto puuttuu" + Chr( 13 ) + "En voihakea" + Chr( 13 )            , ;
         Chr( 13 ) + "Indeksikentt ei lydy" + Chr( 13 ) + "En voihakea" + Chr( 13 )        , ;
         Chr( 13 ) + "En voi hakea memo" + Chr( 13 ) + "tai loogisen kentn mukaan" + Chr( 13 )       , ;
         Chr( 13 ) + "Tietue ei lydy" + Chr( 13 ), ;
         Chr( 13 ) + "Liian monta saraketta" + Chr( 13 ) + "raportti ei mahdu sivulle" + Chr( 13 ) }

      _HMG_aABMLangLabel  := { "Tietue"              , ;
         "Tietue lukumr"    , ;
         "       (Uusi)"       , ;
         "      (Korjaa)"      , ;
         "Anna tietue numero"  , ;
         "Hae"                 , ;
         "Hae teksti"          , ;
         "Hae pivys"         , ;
         "Hae numero"          , ;
         "Raportti mritys"   , ;
         "Raportti sarake"     , ;
         "Sallitut sarakkeet"  , ;
         "Alku tietue"         , ;
         "Loppu tietue"        , ;
         "Raportti "           , ;
         "Pvm:"                , ;
         "Alku tietue:"        , ;
         "Loppu tietue:"       , ;
         "Lajittelu:"         , ;
         "Kyll"                 , ;
         "Ei"                  , ;
         "Sivu "               , ;
         " / "                 }

      _HMG_aABMLangButton := { "Sulje"    , ;
         "Uusi"     , ;
         "Korjaa"   , ;
         "Poista"   , ;
         "Hae"      , ;
         "Mene"     , ;
         "Raportti" , ;
         "Ensimminen" , ;
         "Edellinen"   , ;
         "Seuraava"    , ;
         "Viimeinen"   , ;
         "Tallenna"    , ;
         "Keskeyt"    , ;
         "Lis"       , ;
         "Poista"      , ;
         "Tulosta"     , ;
         "Sulje"     }
      _HMG_aABMLangError  := { "EDIT, tyalue puuttuu"   , ;
         "EDIT, tyalueella yli 16 kentt", ;
         "EDIT, pivitysalue ylitys (raportoi virhe)"      , ;
         "EDIT, tapahtuma numero ylitys (raportoi virhe)" , ;
         "EDIT, lista tapahtuma numero ylitys (raportoi virhe)" }

      // EDIT EXTENDED

      _HMG_aLangButton := {            ;
         " Sulje",            ; // 1
      " Uusi",             ; // 2
      " Muuta",            ; // 3
      " Poista",           ; // 4
      " Hae",              ; // 5
      " Tulosta",          ; // 6
      " Keskeyt",         ; // 7
      " Ok",               ; // 8
      " Kopioi",           ; // 9
      " Aktivoi suodin",   ; // 10
      " Deaktivoi suodin", ; // 11
      " Restore",          ; // 12
		"Retry"              } // 13

      _HMG_aLangLabel := {                        ;
         "Ei mitn",                         ; // 1
      "Tietue",                       ; // 2
      "Yhteens",                        ; // 3
      "Aktiivinen lajittelu",                 ; // 4
      "Optiot",                      ; // 5
      "Uusi tietue",                   ; // 6
      "Muuta tietue",                ; // 7
      "Valitse tietue",                ; // 8
      "Hae tietue",                  ; // 9
      "Tulostus optiot",                ; // 10
      "Valittavat kentt",               ; // 11
      "Tulostettavat kentt",              ; // 12
      "Valittavat tulostimet",           ; // 13
      "Ensim. tulostettava tietue",        ; // 14
      "Viim. tulostettava tietue",         ; // 15
      "Poista tietue",                ; // 16
      "Esikatselu",                      ; // 17
      "Nyt sivujen miniatyyrit",         ; // 18
      "Suodin ehto: ",           ; // 19
      "Suodatettu: ",                   ; // 20
      "Suodatus Optiot" ,           ; // 21
      "Tietokanta kentt" ,             ; // 22
      "Vertailu operaattori",        ; // 23
      "Suodatus arvo",                 ; // 24
      "Valitse suodatus kentt",       ; // 25
      "Valitse vertailu operaattori", ; // 26
      "Yht kuin",                        ; // 27
      "Erisuuri kuin",                    ; // 28
      "Isompi kuin",                 ; // 29
      "Pienempi kuin",                   ; // 30
      "Isompi tai sama kuin",        ; // 31
      "Pienempi tai sama kuin"           } // 32

      _HMG_aLangUser := { ;
         CRLF + "Tyalue ei lydy.   "  + CRLF + "Valitse tyaluetta ennenkun kutsut Edit  " + CRLF,       ; // 1
      "Anna kentt arvo (teksti)",                                  ; // 2
      "Anna kentt arvo (numeerinen)",                                  ; // 3
      "Valitse pivys",                            ; // 4
      "Tarkista tosi arvo",                     ; // 5
      "Anna kentt arvo",                    ; // 6
      "Valitse joku tietue ja paina OK",                                     ; // 7
      CRLF + "Olet poistamassa aktiivinen tietue   " + CRLF + "Oletko varma?    " + CRLF,                  ; // 8
      CRLF + "Ei aktiivista lajittelua   " + CRLF + "Valitse lajittelu   " + CRLF,                            ; // 9
      CRLF + "En voi hakea memo tai loogiseten kenttien perusteella  " + CRLF, ; // 10
      CRLF + "Tietue ei lydy   " + CRLF,                                                ; // 11
      "Valitse listaan listtvt kentt",                                                    ; // 12
      "Valitse EI listtvt kentt",                                        ; // 13
      "Valitse tulostin",                   ; // 14
      "Paina nppin listksesi kentt",                                                                  ; // 15
      "Paina nppin poistaaksesi kentt",                                                       ; //16
      "Paina nppin valittaaksesi ensimminen tulostettava tietue",  ; // 17
      "Paina nppin valittaaksesi viimeinen tulostettava tietue",   ; // 18
      CRLF + "Ei lis kentti   " + CRLF,                                 ; // 19
      CRLF + "Valitse ensin listtv kentt   " + CRLF,                                                           ; //20
      CRLF + "EI Lis ohitettavia kentti   " + CRLF, ; // 21
      CRLF + "Valitse ensin ohitettava kentt   " + CRLF,                                                            ;//22
      CRLF + "Et valinnut kentti   " + CRLF + "Valitse tulosteen kentt   " + CRLF,   ; // 23
      CRLF + "Liikaa kentti   " + CRLF + "Vhenn kentt lukumr   " + CRLF, ; // 24
      CRLF + "Tulostin ei valmiina   " + CRLF,                                                  ; // 25
      "Lajittelu",             ; // 26
      "Tietueesta",              ; // 27
      "Tietueeseen",                  ; // 28
      "Kyll",                ; // 29
      "EI",       ; // 30
      "Sivu:",          ; // 31
      CRLF + "Valitse tulostin   " + CRLF,                                       ; // 32
      "Lajittelu",            ; // 33
      CRLF + "Aktiivinen suodin olemassa    " + CRLF,                                                          ; // 34
      CRLF + "En voi suodattaa memo kentti    " + CRLF, ;// 35
      CRLF + "Valitse suodattava kentt    " + CRLF,                                                           ; // 36
      CRLF + "Valitse suodatus operaattori    " + CRLF,                                                             ; //37
      CRLF + "Anna suodatusarvo    " + CRLF,                                         ; // 38
      CRLF + "Ei aktiivisia suotimia    " + CRLF,                                              ; // 39
      CRLF + "Poista suodin?   " + CRLF,                                        ; // 40
      CRLF + "Tietue lukittu    " + CRLF,                                                              ; // 41
      CRLF + "Palautatko poistetun tietueen   " + CRLF + "Oletko varma?    " + CRLF  } // 42

   CASE cLang == "NL"  // Dutch
      /////////////////////////////////////////////////////////////
      // DUTCH
      ////////////////////////////////////////////////////////////

      // MISC MESSAGES

      _HMG_MESSAGE [1] := 'Weet u het zeker?'
      _HMG_MESSAGE [2] := 'Sluit venster'
      _HMG_MESSAGE [3] := 'Sluiten niet toegestaan'
      _HMG_MESSAGE [4] := 'Programma is al actief'
      _HMG_MESSAGE [5] := 'Bewerken'
      _HMG_MESSAGE [6] := 'Ok'
      _HMG_MESSAGE [7] := 'Annuleren'
      _HMG_MESSAGE [8] := 'Apply'
      _HMG_MESSAGE [9] := 'Pag.'
      _HMG_MESSAGE [10] := 'Aandacht'
      _HMG_MESSAGE [11] := 'Informatie'
      _HMG_MESSAGE [12] := 'Hou op'

      // BROWSE

      _HMG_BRWLangButton := { "Toevoegen"  , ;
         "Bewerken"      , ;
         "&Annuleer"     , ;
         "&OK"           }
      _HMG_BRWLangError  := { "Scherm: ", ;
         " is niet gedefinieerd. Programma beindigd"           , ;
         "MiniGUI fout", ;
         "Control: ", ;
         " Van ", ;
         " Is al gedefinieerd. Programma beindigd"                   , ;
         "Browse: Type niet toegestaan. Programma beindigd"          , ;
         "Browse: Toevoegen-methode kan niet worden gebruikt voor velden die niet bij het Browse werkgebied behoren. Programma beindigd", ;
         "Regel word al veranderd door een andere gebruiker"          , ;
         "Waarschuwing"                                               , ;
         "Onjuiste invoer"                                            }

      _HMG_BRWLangMessage := { 'Weet u het zeker?' , 'Verwijder regel' }

      // EDIT

      _HMG_aABMLangUser   := { Chr( 13 ) + "Verwijder regel" + Chr( 13 ) + "Weet u het zeker ?" + Chr( 13 )    , ;
         Chr( 13 ) + "Index bestand is er niet" + Chr( 13 ) + "Kan niet zoeken" + Chr( 13 )          , ;
         Chr( 13 ) + "Kan index veld niet vinden" + Chr( 13 ) + "Kan niet zoeken" + Chr( 13 )        , ;
         Chr( 13 ) + "Kan niet zoeken op" + Chr( 13 ) + "Memo of logische velden" + Chr( 13 )        , ;
         Chr( 13 ) + "Regel niet gevonden" + Chr( 13 ) , ;
         Chr( 13 ) + "Te veel rijen" + Chr( 13 ) + "Het rapport past niet op het papier" + Chr( 13 ) }

      _HMG_aABMLangLabel  := { "Regel"     , ;
         "Regel aantal"          , ;
         "       (Nieuw)"        , ;
         "      (Bewerken)"      , ;
         "Geef regel nummer"     , ;
         "Vind"                  , ;
         "Zoek tekst"            , ;
         "Zoek datum"            , ;
         "Zoek nummer"           , ;
         "Rapport definitie"     , ;
         "Rapport rijen"         , ;
         "Beschikbare rijen"     , ;
         "Eerste regel"          , ;
         "Laatste regel"         , ;
         "Rapport van "          , ;
         "Datum:"                , ;
         "Eerste regel:"         , ;
         "Laatste tegel:"        , ;
         "Gesorteerd op:"        , ;
         "Ja"                    , ;
         "Nee"                   , ;
         "Pagina "               , ;
         " van "                 }

      _HMG_aABMLangButton := { "Sluiten"   , ;
         "Nieuw"                 , ;
         "Bewerken"              , ;
         "Verwijderen"           , ;
         "Vind"                  , ;
         "Ga naar"               , ;
         "Rapport"               , ;
         "Eerste"                , ;
         "Vorige"                , ;
         "Volgende"              , ;
         "Laatste"               , ;
         "Bewaar"                , ;
         "Annuleren"             , ;
         "Voeg toe"              , ;
         "Verwijder"             , ;
         "Print"                 , ;
         "Sluiten"               }
      _HMG_aABMLangError  := { "BEWERKEN, werkgebied naam bestaat niet", ;
         "BEWERKEN, dit werkgebied heeft meer dan 16 velden", ;
         "BEWERKEN, ververs manier buiten bereik (a.u.b. fout melden)"           , ;
         "BEWERKEN, hoofd gebeurtenis nummer buiten bereik (a.u.b. fout melden)" , ;
         "BEWERKEN, list gebeurtenis nummer buiten bereik (a.u.b. fout melden)"  }

      // EDIT EXTENDED
      _HMG_aLangButton := {            ;
         "&Sluiten",          ; // 1
      "&Nieuw",            ; // 2
      "&Aanpassen",        ; // 3
      "&Verwijderen",      ; // 4
      "&Vind",             ; // 5
      "&Print",            ; // 6
      "&Annuleren",        ; // 7
      "&Ok",               ; // 8
      "&Kopieer",          ; // 9
      "&Activeer filter",  ; // 10
      "&Deactiveer filter",; // 11
      "&Restore",          ; // 12
		"Retry"              } // 13

      _HMG_aLangLabel := {                            ;
         "Geen",                             ; // 1
      "Regel",                            ; // 2
      "Totaal",                           ; // 3
      "Actieve volgorde",                 ; // 4
      "Opties",                           ; // 5
      "Nieuw regel",                      ; // 6
      "Aanpassen regel",                  ; // 7
      "Selecteer regel",                  ; // 8
      "Vind regel",                       ; // 9
      "Print opties",                     ; //10
      "Beschikbare velden",               ; //11
      "Velden te printen",                ; //12
      "Beschikbare printers",             ; //13
      "Eerste regel te printen",          ; //14
      "Laatste regel te printen",         ; //15
      "Verwijder regel",                  ; //16
      "Voorbeeld",                        ; //17
      "Laat pagina klein zien",           ; //18
      "Filter condities: ",               ; //19
      "Gefilterd: ",                      ; //20
      "Filter opties" ,                   ; //21
      "Database velden" ,                 ; //22
      "Vergelijkings operator",           ; //23
      "Filter waarde",                    ; //24
      "Selecteer velden om te filteren",  ; //25
      "Selecteer vergelijkings operator", ; //26
      "Gelijk",                           ; //27
      "Niet gelijk",                      ; //28
      "Groter dan",                       ; //29
      "Kleiner dan",                      ; //30
      "Groter dan of gelijk aan",         ; //31
      "Kleiner dan of gelijk aan"         } //32
      _HMG_aLangUser := { ;
         CRLF + "Kan geen actief werkgebied vinden   "  + CRLF + "Selecteer A.U.B. een actief werkgebied voor BEWERKEN aan te roepen   " + CRLF, ; // 1
      "Geef de veld waarde (een tekst)", ; // 2
      "Geef de veld waarde (een nummer)", ; // 3
      "Selecteer de datum", ; // 4
      "Controleer voor geldige waarde", ; // 5
      "Geef de veld waarde", ; // 6
      "Selecteer een regel en druk op OK", ; // 7
      CRLF + "Je gaat het actieve regel verwijderen  " + CRLF + "Zeker weten?    " + CRLF, ; // 8
      CRLF + "Er is geen actieve volgorde " + CRLF + "Selecteer er A.U.B. een   " + CRLF, ; // 9
      CRLF + "Kan niet zoeken in memo of logische velden   " + CRLF, ; // 10
      CRLF + "Regel niet gevonden   " + CRLF, ; // 11
      "Selecteer het veld om in de lijst in te sluiten", ; // 12
      "Selecteer het veld om uit de lijst te halen", ; // 13
      "Selecteer de printer", ; // 14
      "Druk op de knop om het veld in te sluiten", ; // 15
      "Druk op de knop om het veld uit te sluiten", ; // 16
      "Druk op de knop om het eerste veld te selecteren om te printen", ; // 17
      "Druk op de knop om het laatste veld te selecteren om te printen", ; // 18
      CRLF + "Geen velden meer om in te sluiten   " + CRLF, ; // 19
      CRLF + "Selecteer eerst het veld om in te sluiten   " + CRLF, ; // 20
      CRLF + "Geen velden meer om uit te sluiten   " + CRLF, ; // 21
      CRLF + "Selecteer eerst het veld om uit te sluiten   " + CRLF, ; // 22
      CRLF + "Je hebt geen velden geselecteerd   " + CRLF + "Selecteer A.U.B. de velden om in te sluiten om te printen   " + CRLF, ; // 23
      CRLF + "Teveel velden   " + CRLF + "Selecteer minder velden   " + CRLF, ; // 24
      CRLF + "Printer niet klaar   " + CRLF, ; // 25
      "Volgorde op", ; // 26
      "Van regel", ; // 27
      "Tot regel", ; // 28
      "Ja", ; // 29
      "Nee", ; // 30
      "Pagina:", ; // 31
      CRLF + "Selecteer A.U.B. een printer " + CRLF, ; // 32
      "Gefilterd op", ; // 33
      CRLF + "Er is een actief filter    " + CRLF, ; // 34
      CRLF + "Kan niet filteren op memo velden    " + CRLF, ; // 35
      CRLF + "Selecteer het veld om op te filteren    " + CRLF, ; // 36
      CRLF + "Selecteer een operator om te filteren    " + CRLF, ; // 37
      CRLF + "Type een waarde om te filteren " + CRLF, ; // 38
      CRLF + "Er is geen actief filter    " + CRLF, ; // 39
      CRLF + "Deactiveer filter?   " + CRLF, ; // 40
      CRLF + "Regel geblokkeerd door een andere gebuiker    " + CRLF, ; // 41
      CRLF + "You are going to restore the deleted record   " + CRLF + "Are you sure?    " + CRLF } // 42

   CASE cLang == "SL"  // Slovenian
      /////////////////////////////////////////////////////////////
      // SLOVENIAN
      ////////////////////////////////////////////////////////////

      // MISC MESSAGES

      _HMG_MESSAGE [1] := 'Ste prepriani ?'
      _HMG_MESSAGE [2] := 'Zapri okno'
      _HMG_MESSAGE [3] := 'Zapiranje ni dovoljeno'
      _HMG_MESSAGE [4] := 'Program je e zagnan'
      _HMG_MESSAGE [5] := 'Popravi'
      _HMG_MESSAGE [6] := 'V redu'
      _HMG_MESSAGE [7] := 'Prekini'
      _HMG_MESSAGE [8] := 'Apply'
      _HMG_MESSAGE [9] := 'Str.'
      _HMG_MESSAGE [10] := 'Attention'
      _HMG_MESSAGE [11] := 'Information'
      _HMG_MESSAGE [12] := 'Stop'

      // BROWSE MESSAGES

      _HMG_BRWLangButton := { "Dodaj" , ;
         "Popravi"        , ;
         "Prekini"        , ;
         "V redu" }

      _HMG_BRWLangError  := { "Window: "                    , ;
         " not defined."     , ;
         "MiniGUI Error"                        , ;
         "Control: "                            , ;
         " Of "                                 , ;
         " Already defined." , ;
         "Type Not Allowed." , ;
         "False WorkArea."   , ;
         "Zapis ureja drug uporabnik"           , ;
         "Opozorilo"                            , ;
         "Narobe vnos" }

      _HMG_BRWLangMessage := { 'Ste prepriani ?' , 'Brii vrstico' }

      // EDIT MESSAGES

      _HMG_aABMLangUser   := { Chr( 13 ) + "Brii vrstico" + Chr( 13 ) + "Ste prepriani ?" + Chr( 13 ) , ;
         Chr( 13 ) + "Manjka indeksna datoteka" + Chr( 13 ) + "Ne morem iskati" + Chr( 13 )       , ;
         Chr( 13 ) + "Ne najdem indeksnega polja" + Chr( 13 ) + "Ne morem iskati" + Chr( 13 )     , ;
         Chr( 13 ) + "Ne morem iskati po" + Chr( 13 ) + "memo ali loginih poljih" + Chr( 13 )    , ;
         Chr( 13 ) + "Ne najdem vrstice" + Chr( 13 )                                        , ;
         Chr( 13 ) + "Preve kolon" + Chr( 13 ) + "Poroilo ne gre na list" + Chr( 13 ) }

      _HMG_aABMLangLabel  := { "Vrstica", ;
         "tevilo vrstic"         , ;
         "       (Nova)"          , ;
         "      (Popravi)"        , ;
         "Vnesi tevilko vrstice" , ;
         "Poii"                 , ;
         "Besedilo za iskanje"    , ;
         "Datum za iskanje"       , ;
         "tevilka za iskanje"    , ;
         "Parametri poroila"     , ;
         "Kolon v poroilu"       , ;
         "Kolon na razpolago"     , ;
         "Zaetna vrstica"        , ;
         "Konna vrstica"         , ;
         "Poroilo za "           , ;
         "Datum:"                 , ;
         "Zaetna vrstica:"       , ;
         "Konna vrstica:"        , ;
         "Urejeno po:"            , ;
         "Ja"                     , ;
         "Ne"                     , ;
         "Stran "                 , ;
         " od "                 }

      _HMG_aABMLangButton := { "Zapri" , ;
         "Nova"              , ;
         "Uredi"             , ;
         "Brii"             , ;
         "Poii"            , ;
         "Pojdi na"          , ;
         "Poroilo"          , ;
         "Prva"              , ;
         "Prejnja"          , ;
         "Naslednja"         , ;
         "Zadnja"            , ;
         "Shrani"            , ;
         "Prekini"           , ;
         "Dodaj"             , ;
         "Odstrani"          , ;
         "Natisni"           , ;
         "Zapri"     }
      _HMG_aABMLangError  := { "EDIT, workarea name missing"              , ;
         "EDIT, this workarea has more than 16 fields"              , ;
         "EDIT, refresh mode out of range (please report bug)"      , ;
         "EDIT, main event number out of range (please report bug)" , ;
         "EDIT, list event number out of range (please report bug)"  }

      // EDIT EXTENDED

      _HMG_aLangButton := { ;
         "&Zapri",             ; // 1
      "&Nova",              ; // 2
      "&Spremeni",          ; // 3
      "&Brii",             ; // 4
      "&Poii",            ; // 5
      "&Natisni",           ; // 6
      "&Prekini",           ; // 7
      "&V redu",            ; // 8
      "&Kopiraj",           ; // 9
      "&Aktiviraj Filter",  ; // 10
      "&Deaktiviraj Filter", ; // 11
      "&Obnovi",             ; // 12
		"Retry"                } // 13


      _HMG_aLangLabel := {                 ;
         "Prazno",                        ; // 1
      "Vrstica",                       ; // 2
      "Skupaj",                        ; // 3
      "Activni indeks",                ; // 4
      "Monosti",                      ; // 5
      "Nova vrstica",                  ; // 6
      "Spreminjaj vrstico",            ; // 7
      "Oznai vrstico",                ; // 8
      "Najdi vrstico",                 ; // 9
      "Monosti tiskanja",             ; // 10
      "Polja na razpolago",            ; // 11
      "Polja za tiskanje",             ; // 12
      "Tiskalniki na razpolago",       ; // 13
      "Prva vrstica za tiskanje",      ; // 14
      "Zadnja vrstica za tiskanje",    ; // 15
      "Brii vrstico",                 ; // 16
      "Pregled",                       ; // 17
      "Mini pregled strani",           ; // 18
      "Pogoj za filter: ",             ; // 19
      "Filtrirano: ",                  ; // 20
      "Monosti filtra" ,              ; // 21
      "Polja v datoteki" ,             ; // 22
      "Operator za primerjavo",        ; // 23
      "Vrednost filtra",               ; // 24
      "Izberi polje za filter",        ; // 25
      "Izberi operator za primerjavo", ; // 26
      "Enako",                         ; // 27
      "Neenako",                       ; // 28
      "Veje od",                      ; // 29
      "Manje od",                     ; // 30
      "Veje ali enako od",            ; // 31
      "Manje ali enako od"            } // 32
      _HMG_aLangUser := { ;
         CRLF + "Can't find an active area.   "  + CRLF + "Please select any area before call EDIT   " + CRLF,    ; // 1
      "Vnesi vrednost (tekst)",                                                                                ; // 2
      "Vnesi vrednost (tevilka)",                                                                             ; // 3
      "Izberi datum",                                                                                          ; // 4
      "Oznai za logini DA",                                                                                  ; // 5
      "Vnesi vrednost",                                                                                        ; // 6
      "Izberi vrstico in pritisni <V redu>",                                                                   ; // 7
      CRLF + "Pobrisali boste trenutno vrstico   " + CRLF + "Ste prepriani?    " + CRLF,                      ; // 8
      CRLF + "Ni aktivnega indeksa   " + CRLF + "Prosimo, izberite ga   " + CRLF,                              ; // 9
      CRLF + "Ne morem iskati po loginih oz. memo poljih   " + CRLF,                                          ; // 10
      CRLF + "Ne najdem vrstice   " + CRLF,                                                                    ; // 11
      "Izberite polje, ki BO vkljueno na listo",                                                              ; // 12
      "Izberite polje, ki NI vkljueno na listo",                                                              ; // 13
      "Izberite tisklanik",                                                                                    ; // 14
      "Pritisnite gumb za vkljuitev polja",                                                                   ; // 15
      "Pritisnite gumb za izkljuitev polja",                                                                  ; // 16
      "Pritisnite gumb za izbor prve vrstice za tiskanje",                                                     ; // 17
      "Pritisnite gumb za izbor zadnje vrstice za tiskanje",                                                   ; // 18
      CRLF + "Ni ve polj za dodajanje   " + CRLF,                                                             ; // 19
      CRLF + "Najprej izberite ppolje za vkljuitev   " + CRLF,                                                ; // 20
      CRLF + "Ni ve polj za izkljuitev   " + CRLF,                                                           ; // 21
      CRLF + "Najprej izberite polje za izkljuitev   " + CRLF,                                                ; // 22
      CRLF + "Niste izbrali nobenega polja   " + CRLF + "Prosom, izberite polje za tiskalnje   " + CRLF,       ; // 23
      CRLF + "Preve polj   " + CRLF + "Zmanjajte tevilo polj   " + CRLF,                                    ; // 24
      CRLF + "Tiskalnik ni pripravljen   " + CRLF,                                                             ; // 25
      "Urejeno po",                                                                                            ; // 26
      "Od vrstice",                                                                                            ; // 27
      "do vrstice",                                                                                            ; // 28
      "Ja",                                                                                                    ; // 29
      "Ne",                                                                                                    ; // 30
      "Stran:",                                                                                                ; // 31
      CRLF + "Izberite tiskalnik   " + CRLF,                                                                   ; // 32
      "Filtrirano z",                                                                                          ; // 33
      CRLF + "Aktivni filter v uporabi    " + CRLF,                                                            ; // 34
      CRLF + "Ne morem filtrirati z memo polji    " + CRLF,                                                    ; // 35
      CRLF + "Izberi polje za filtriranje    " + CRLF,                                                         ; // 36
      CRLF + "Izberi operator za filtriranje    " + CRLF,                                                      ; // 37
      CRLF + "Vnesi vrednost za filtriranje    " + CRLF,                                                       ; // 38
      CRLF + "Ni aktivnega filtra    " + CRLF,                                                                 ; // 39
      CRLF + "Deaktiviram filter?   " + CRLF,                                                                  ; // 40
      CRLF + "Vrstica zaklenjena - uporablja jo drug uporabnik    " + CRLF,                                    ; // 41
      CRLF + "Obnovili boste pobrisano vrstico   " + CRLF + "Ste prepriani?    " + CRLF                       } // 42

   CASE cLang == "SK"  // Slovak
      /////////////////////////////////////////////////////////////
      // SLOVAK
      ////////////////////////////////////////////////////////////

      // MISC MESSAGES

      _HMG_MESSAGE [1] := 'Ste si ist()?'
      _HMG_MESSAGE [2] := 'Zatvor okno'
      _HMG_MESSAGE [3] := 'Zatvorenie nedovolen'
      _HMG_MESSAGE [4] := 'Program u be'
      _HMG_MESSAGE [5] := 'prava'
      _HMG_MESSAGE [6] := 'Ok'
      _HMG_MESSAGE [7] := 'Storno'
      _HMG_MESSAGE [8] := 'Aplikuj'
      _HMG_MESSAGE [9] := 'Str.'
      _HMG_MESSAGE [10] := 'Attention'
      _HMG_MESSAGE [11] := 'Information'
      _HMG_MESSAGE [12] := 'Stop'

      // BROWSE MESSAGES

      _HMG_BRWLangButton := { "P&rida"  , ;
         "&Upravi" , ;
         "&Storno" , ;
         "&OK"       }
      _HMG_BRWLangError  := { "Okno: "             , ;
         " nedefinovan. Program ukonen.", ;
         "MiniGUI Error"                   , ;
         "Prvok: "                         , ;
         " z "                             , ;
         " u definovan. Program ukonen"                  , ;
         "Prezeranie: Nedovolen typ. Program ukonen"      , ;
         "Prezeranie: Clauzula 'Prida' je nepouiten so stpcami nepatriacimi do pracovnej oblasti browse. Program ukonen", ;
         "Zznam upravuje in uvate"                      , ;
         "Varovanie"                                         , ;
         "Chybn vstup"                                         }
      _HMG_BRWLangMessage := { 'Ste si ist() ?' , 'Zmaza zznam' }

      // EDIT MESSAGES

      _HMG_aABMLangUser   := { Chr( 13 ) + "Zmaza zznam." + Chr( 13 ) + "Ste si ist()?" + Chr( 13 )                  , ;
         Chr( 13 ) + "Chba indexov sbor!" + Chr( 13 ) + "Nemem hada." + Chr( 13 )            , ;
         Chr( 13 ) + "Nebol njden index!" + Chr( 13 ) + "Nemem hada." + Chr( 13 )        , ;
         Chr( 13 ) + "Nemem hada poda" + Chr( 13 ) + "stpca typu memo alebo logical." + Chr( 13 )       , ;
         Chr( 13 ) + "Zznam nebol njden!" + Chr( 13 )                                        , ;
         Chr( 13 ) + "Prli vea stpcov!" + Chr( 13 ) + "Zostava sa nezmest na plochu" + Chr( 13 ) }

      _HMG_aABMLangLabel  := { "Zznam"   , ;
         "Poet zznamov"     , ;
         "      (Nov)"       , ;
         "   (Upravi)"       , ;
         "Zadajte slo zznamu", ;
         "Hadaj"             , ;
         "Hadan text"       , ;
         "Hadan dtum"      , ;
         "Hadan slo"      , ;
         "Defincia zostavy"  , ;
         "Stpce zostavy"     , ;
         "Dostupn stpce"    , ;
         "Prv zznam"        , ;
         "Posledn zznam"    , ;
         "Zostava "           , ;
         "Dtum:"             , ;
         "Prv zznam:"       , ;
         "Posledn zznam:"   , ;
         "Zoraden poda:"    , ;
         "no"                , ;
         "Nie"                , ;
         "Strana "            , ;
         " z "                   }

      _HMG_aABMLangButton := { "Zatvor" , ;
         "Nov"             , ;
         "prava"           , ;
         "Zru"             , ;
         "Njdi"            , ;
         "Cho na"          , ;
         "Zostava"          , ;
         "Prv"             , ;
         "Predol"         , ;
         "a"            , ;
         "Posledn"         , ;
         "Ulo"             , ;
         "Storno"           , ;
         "Pridaj"           , ;
         "Odstr"          , ;
         "Tla"             , ;
         "Zatvor"     }
      _HMG_aABMLangError  := { "EDIT, chba meno pracovnej oblasti"                              , ;
         "EDIT, pracovn oblas m viac ako 16 stpcov"              , ;
         "EDIT, refresh mode je mimo rozsah (prosm, nahlste chybu)"      , ;
         "EDIT, slo hlavnej udalosti mimo rozsah (prosm, nahlste chybu)" , ;
         "EDIT, slo udalosti, list mimo rozsah (prosm, nahlste chybu)"  }

      // EDIT EXTENDED

      _HMG_aLangButton := { ;
         "&Zatvor",           ; // 1
      "&Nov",             ; // 2
      "&prava",           ; // 3
      "Zr&u",             ; // 4
      "N&jdi",            ; // 5
      "&Tla",             ; // 6
      "&Storno",           ; // 7
      "&Ok",               ; // 8
      "&Kopruj",          ; // 9
      "Zapni &filter",     ; // 10
      "&Vypni filter",     ; // 11
      "O&bnov",            ; // 12
		"Retry"             } // 13

      _HMG_aLangLabel := {                     ;
         "iadna",                         ; // 1
      "Veta",                           ; // 2
      "Suma",                           ; // 3
      "Aktvne zoradenie",              ; // 4
      "Monosti",                       ; // 5
      "Nov zznam",                    ; // 6
      "Uprav zznam",                   ; // 7
      "Vyber zznam",                   ; // 8
      "Njdi zznam",                   ; // 9
      "Tla volby",                     ; // 10
      "Dostupn stpce",                ; // 11
      "Stpce pre tla",                ; // 12
      "Dostupn tlaiarne",             ; // 13
      "Prv zznam pre tla",           ; // 14
      "Posledn zznam pre tla",       ; // 15
      "Vyma zznam",                   ; // 16
      "Nhad",                         ; // 17
      "Zobraz miniatry strn",         ; // 18
      "Podmienky filtra: ",             ; // 19
      "Filtrovan: ",                   ; // 20
      "Monosti filtra",                ; // 21
      "Stpce dtabzy",                ; // 22
      "Opertor porovnanie",            ; // 23
      "Hodnota filtra",                 ; // 24
      "Vber pola do filtra",           ; // 25
      "Vber opertora porovnania",     ; // 26
      "rovn sa",                       ; // 27
      "nerovn sa",                     ; // 28
      "v ako",                      ; // 29
      "men ako",                      ; // 30
      "v alebo rovn ako",          ; // 31
      "men alebo rovn ako"          }  // 32
      _HMG_aLangUser := { ;
         CRLF + "Nenjden aktvna oblas   "  + CRLF + "Vyberte prosm pred volanm EDIT hociktor oblas   " + CRLF,     ; // 1
      "Zadajte hodnotu stpca (ubovoln text)",                                                          ; // 2
      "Zadajte hodnotu stpca (ubovoln slo)",                                                         ; // 3
      "Vyberte dtum",                                                                                    ; // 4
      "Zakrtnite pre hodnotu 'true'",                                                                    ; // 5
      "Zadajte hodnotu stpca",                                                                           ; // 6
      "Vyberte niektor vetu a stlate OK",                                                               ; // 7
      CRLF + "Chcete zrui tento zznam   " + CRLF + "Ste si ist()?    " + CRLF,                       ; // 8
      CRLF + "iadn zoradenie nie je aktvn   " + CRLF + "Prosm vyberte jedno   " + CRLF,              ; // 9
      CRLF + "Nemem hada poda stpca typu memo alebo logical " + CRLF,                               ; // 10
      CRLF + "Nenjden zznam   " + CRLF,                                                                ; // 11
      "Vyberte stpec k vloeniu do zoznamu",                                                             ; // 12
      "Vyberte stpec k odstrneniu zo zoznamu",                                                          ; // 13
      "Vyberte tlaiare",                                                                                ; // 14
      "Stlate tlaidlo zaloenia stpca",                                                                ; // 15
      "Stlate tlaidlo odstrnenia stpca",                                                              ; // 16
      "Stlate tlaidlo - Prv zznam pre tla",                                                          ; // 17
      "Stlate tlaidlo - Posledn zznam ptre tla",                                                     ; // 18
      CRLF + "Nie je dostupn stpec k zaloeniu   " + CRLF,                                              ; // 19
      CRLF + "Najprv vyberte stpec k zaloeniu   " + CRLF,                                               ; // 20
      CRLF + "al stpec nemete odstrni   " + CRLF,                                                 ; // 21
      CRLF + "Najprv vyberte n. stpec k odstrneniu   " + CRLF,                                          ; // 22
      CRLF + "Nevybrali ste ani jeden stlpec   " + CRLF + "Prosm vyberte stpce pre tla   " + CRLF,     ; // 23
      CRLF + "Prli vea stpcov   " + CRLF + "odstrte niekter stpce   " + CRLF,                     ; // 24
      CRLF + "Tlaiare nie je pripraven   " + CRLF,                                                     ; // 25
      "Zoraden poda",                                                                                   ; // 26
      "Od zznamu",                                                                                       ; // 27
      "Po zznam",                                                                                        ; // 28
      "no",                                                                                              ; // 29
      "Nie",                                                                                              ; // 30
      "Strana:",                                                                                          ; // 31
      CRLF + "Vyberte si tlaiare   " + CRLF,                                                            ; // 32
      "Filtrovan poda",                                                                                 ; // 33
      CRLF + "Aktivn filter    " + CRLF,                                                                 ; // 34
      CRLF + "Nemete filtrova poda stpca typu memo    " + CRLF,                                      ; // 35
      CRLF + "Vyberte stpec do filtra    " + CRLF,                                                       ; // 36
      CRLF + "Vyberte opertor do filtra    " + CRLF,                                                     ; // 37
      CRLF + "Zadajte hodnotu do filtra    " + CRLF,                                                      ; // 38
      CRLF + "iadny aktvny filter    " + CRLF,                                                          ; // 39
      CRLF + "Deaktivova filter?   " + CRLF,                                                             ; // 40
      CRLF + "Zznam blokovan inm uvateom  " + CRLF,                                                 ; // 41
      CRLF + "Chcete obnovi vymazan zznamy   " + CRLF + "Ste si ist()?    " + CRLF                   } // 42

   CASE cLang == "HU"  // Hungarian
      /////////////////////////////////////////////////////////////
      // HUNGARIAN
      ////////////////////////////////////////////////////////////

      // MISC MESSAGES

      _HMG_MESSAGE [1] := 'Biztos benne?'
      _HMG_MESSAGE [2] := 'Zrja be az ablakot'
      _HMG_MESSAGE [3] := 'Bezrs tiltva'
      _HMG_MESSAGE [4] := 'Program mr fut'
      _HMG_MESSAGE [5] := 'Szerkeszts'
      _HMG_MESSAGE [6] := 'Ok'
      _HMG_MESSAGE [7] := 'Mgse'
      _HMG_MESSAGE [8] := 'Apply'
      _HMG_MESSAGE [9] := 'Old.'
      _HMG_MESSAGE [10] := 'Attention'
      _HMG_MESSAGE [11] := 'Information'
      _HMG_MESSAGE [12] := 'Stop'

      // BROWSE MESSAGES

      _HMG_BRWLangButton := { "Hozzad"  , ;
         "Szerkeszts" , ;
         "&Mgse" , ;
         "&OK"       }
      _HMG_BRWLangError  := { "Ablak: "                            , ;
         " nem definilt. Program vge" , ;
         "MiniGUI Error"                   , ;
         "Elem: "                         , ;
         " ebbl "                             , ;
         " mr definilt. Program vge"                  , ;
         "Bngsz: Tiltott tpus. Program vge"      , ;
         "Bngsz: Hozzad frzis nem hasznlhat olyan mezre, mely nincs a Bngsz munkaterletben. Program vge", ;
         "A rekordot egy msik felhasznl szerkeszti"                      , ;
         "Figyelmeztets"                                         , ;
         "Hibs adat"                                         }
      _HMG_BRWLangMessage := { 'Biztos benne ?' , 'Rekord trlse' }

      // EDIT MESSAGES

      _HMG_aABMLangUser   := { Chr( 13 ) + "Rekord trlse" + Chr( 13 ) + "Biztos benne ?" + Chr( 13 )                  , ;
         Chr( 13 ) + "Hinyz index llomny" + Chr( 13 ) + "Lehetetlen a keress" + Chr( 13 )            , ;
         Chr( 13 ) + "Hinyz index mez" + Chr( 13 ) + "Lehetetlen a keress" + Chr( 13 )        , ;
         Chr( 13 ) + "Lehetetlen a keress a kv.alapjn" + Chr( 13 ) + "mez memo vagy logikai" + Chr( 13 )       , ;
         Chr( 13 ) + "Nincs rekord" + Chr( 13 )                                        , ;
         Chr( 13 ) + "Tl sok oszlop" + Chr( 13 ) + "A jelents nem fr el a felleten" + Chr( 13 ) }

      _HMG_aABMLangLabel  := { "Rekord"         , ;
         "Rekord szmll"          , ;
         "           (j)"          , ;
         "  (Szerkeszts)"          , ;
         "Adja meg a rekord szmt" , ;
         "Keresd"                   , ;
         "Keresett szveg"          , ;
         "Keresett dtum"           , ;
         "Keresett szm"            , ;
         "A jelents meghatrozsa" , ;
         "A jelents oszlopai"      , ;
         "Hasznlhat oszlopok"     , ;
         "Kezd rekord"             , ;
         "Utols rekord"            , ;
         "Jelents a "              , ;
         "Dtum:"                   , ;
         "Kezd rekord:"            , ;
         "Utols rekord:"           , ;
         "Rendezve:"                , ;
         "Igen"                     , ;
         "Nem"                      , ;
         "Oldal "                   , ;
         " / "                       }

      _HMG_aABMLangButton := { "Zrd be" , ;
         "j"            , ;
         "Szerkeszts"   , ;
         "Trld"        , ;
         "Keresd"        , ;
         "Menj a"        , ;
         "Jelents"      , ;
         "Els"          , ;
         "Elz"         , ;
         "Kvetkez"     , ;
         "Utols"        , ;
         "Mentsd"        , ;
         "Mgse"         , ;
         "Add"           , ;
         "Eltvolt"     , ;
         "Nyomtats"     , ;
         "Zrd be"     }
      _HMG_aABMLangError  := { "SZERKESZTS, hibs a munkaterlet neve"                              , ;
         "SZERKESZTS, a munkaterletnek tbb mint 16 mezt tartalmaz"              , ;
         "SZERKESZTS, a frisstsi md rtken kvl (krem, jelentse a hibt)"      , ;
         "SZERKESZTS, a f esemny szma rtken kvl (krem, jelentse a hibt)" , ;
         "SZERKESZTS, a lista esemny szma rtken kvl(krem, jelentse a hibt)"  }

      // EDIT EXTENDED

      _HMG_aLangButton := { ;
         "&Zrd be",               ; // 1
      "&j",                    ; // 2
      "M&dosts",             ; // 3
      "&Trld",                ; // 4
      "&Keresd",                ; // 5
      "&Nyomtats",             ; // 6
      "&Mgse",                 ; // 7
      "&Ok",                    ; // 8
      "Mso&l",                 ; // 9
      "Szr &bekapcsolsa",    ; // 10
      "Szr ki&kapcsolsa",    ; // 11
      "&Visszahv",             ; // 12
		"Retry"                   } // 13

      _HMG_aLangLabel := {                            ;
         "nincs",                                 ; // 1
      "rekord",                                ; // 2
      "sszeg",                                ; // 3
      "Aktv rendezs",                        ; // 4
      "Opcik",                                ; // 5
      "j rekord",                             ; // 6
      "Rekord mdostsa",                     ; // 7
      "Rekord kivlasztsa",                   ; // 8
      "Rekord keresse",                       ; // 9
      "Nyomtatsi opcik",                     ; // 10
      "Hasznlhat mezk",                     ; // 11
      "Nyomtathat mezk",                     ; // 12
      "Elrhet nyomtatk",                    ; // 13
      "Nyomtats els rekordja",               ; // 14
      "Nyomtats utols rekordja",             ; // 15
      "Rekord trlse",                        ; // 16
      "Elnzet",                              ; // 17
      "Oldalak miniatraknt",                 ; // 18
      "Szr felttele: ",                     ; // 19
      "Szrve: ",                              ; // 20
      "Szr opcii",                          ; // 21
      "Adatbzis mezi",                       ; // 22
      "sszehasonlt opertor",               ; // 23
      "Szr rtke",                          ; // 24
      "Mezk kivlasztsa a szrhz",         ; // 25
      "sszehasonlt operctor kivlasztsa",  ; // 26
      "egyenl",                               ; // 27
      "nem egyenl",                           ; // 28
      "nagyobb mint",                          ; // 29
      "kisebb mint",                           ; // 30
      "nagyobb vagy egyenl mint",             ; // 31
      "kisebb vagy egyenl mint"          }      // 32
      _HMG_aLangUser := { ;
         CRLF + "Nincs aktv munkaterlet   "  + CRLF + "Krem vlasszon egy munkaterletet a SZERKESZTS hvsa eltt   " + CRLF,     ; // 1
      "Adja meg a mez rtkt (szveget)",                                                                 ; // 2
      "Adja meg a mez rtkt (szmot)",                                                                   ; // 3
      "Vlasszon dtumot",                                                                                  ; // 4
      "Pipzza az igaz rtket",                                                                            ; // 5
      "Adja meg a mez rtkt",                                                                            ; // 6
      "Vlasszon egy rekordot s nyomjon OK",                                                               ; // 7
      CRLF + "Aktv rekordot kvnja trlni   " + CRLF + "Biztos benne?    " + CRLF,                       ; // 8
      CRLF + "Nincs aktv sorba rendezs   " + CRLF + "Krem vlasszon egyet   " + CRLF,                    ; // 9
      CRLF + "Nem kereshetek memo vagy logikai mez utn   " + CRLF,                                        ; // 10
      CRLF + "Nincs rekord   " + CRLF,                                                                      ; // 11
      "Vlasszon a listhoz hozzadand mezt",                                                             ; // 12
      "Vlasszon a listbl kiveend mezt",                                                                ; // 13
      "Vlasszon nyomtatt",                                                                                ; // 14
      "Mez hozzads gombot nyomja meg",                                                                   ; // 15
      "Mez trlse gombot nyomja meg",                                                                     ; // 16
      "A nyomtats els rekordja gombot nyomja meg",                                                        ; // 17
      "A nyomtats utols rekordja gombot nyomja meg",                                                      ; // 18
      CRLF + "Nincs tbb hozzadhat mez   " + CRLF,                                                       ; // 19
      CRLF + "Elszr hozzadand mezt vlasszon   " + CRLF,                                               ; // 20
      CRLF + "Nincs tbb kivehet mez   " + CRLF,                                                          ; // 21
      CRLF + "Elszr az n. kiveend mezt vlassza ki   " + CRLF,                                          ; // 22
      CRLF + "Egy mezt sem vlasztott ki   " + CRLF + "Krem vlassza ki a nyomtatand mezket   " + CRLF, ; // 23
      CRLF + "Tl sok mez   " + CRLF + "Reduklja a mezk szmt   " + CRLF,                               ; // 24
      CRLF + "A nyomtat nem ksz   " + CRLF,                                                               ; // 25
      "Rendezve",                                                                                           ; // 26
      "Rekordtl",                                                                                          ; // 27
      "Rekordig",                                                                                           ; // 28
      "Igen",                                                                                               ; // 29
      "Nem",                                                                                                ; // 30
      "Oldal:",                                                                                             ; // 31
      CRLF + "Krem vlasszon nyomtatt   " + CRLF,                                                         ; // 32
      "Szrve",                                                                                             ; // 33
      CRLF + "Ez az aktv szr   " + CRLF,                                                                 ; // 34
      CRLF + "Nem szrhet memo mez alapjn   " + CRLF,                                                   ; // 35
      CRLF + "Vlasszon mezt a szrhz   " + CRLF,                                                        ; // 36
      CRLF + "Vlasszon opertort a szrhz   " + CRLF,                                                    ; // 37
      CRLF + "Adjon rtket  a szrhz   " + CRLF,                                                         ; // 38
      CRLF + "Nincs egy aktv szr   " + CRLF,                                                             ; // 39
      CRLF + "A szr deaktivlsa?   " + CRLF,                                                             ; // 40
      CRLF + "A rekord blokkolva msik felhasznl ltal   " + CRLF,                                        ; // 41
      CRLF + "Trlt rekordokat kvn visszahvni   " + CRLF + "Biztos benne ?    " + CRLF                  } // 42

   CASE cLang == "EL"  // Greek - Ellinika
      /////////////////////////////////////////////////////////////
      // GREEK -  - EL
      /////////////////////////////////////////////////////////////

      // MISC MESSAGES (GREEK EL)

      _HMG_MESSAGE [1] := ' ?'
      _HMG_MESSAGE [2] := ' '
      _HMG_MESSAGE [3] := '   '
      _HMG_MESSAGE [4] := '   '
      _HMG_MESSAGE [5] := '.'
      _HMG_MESSAGE [6] := 'Ok'
      _HMG_MESSAGE [7] := ''
      _HMG_MESSAGE [8] := ''
      _HMG_MESSAGE [9] := '.'
      _HMG_MESSAGE [10] := ''
      _HMG_MESSAGE [11] := ''
      _HMG_MESSAGE [12] := 'Stop'

      // BROWSE MESSAGES (GREEK EL)

      _HMG_BRWLangButton := { "&"  , ;
         "&"    , ;
         "&"  , ;
         "&OK"       }

      _HMG_BRWLangError  := { " window: "                                           , ;
         "   .   "          , ;
         "MiniGUI Error"                                         , ;
         "Control: "                                             , ;
         " Of "                                                  , ;
         "   .   "          , ;
         "Browse:   .   "          , ;
         "Browse: Append Clause Can't Be Used With Fields Not Belonging To Browse WorkArea.", ;
         "     "                , ;
         ""                                               , ;
         "  "                                          }

      _HMG_BRWLangMessage := { '  ?' , ' ' }

      // EDIT MESSAGES (GREEK - )

      _HMG_aABMLangUser   := { Chr( 13 ) + " " + Chr( 13 ) + " ?" + Chr( 13 )    , ;
         Chr( 13 ) + "   " + Chr( 13 ) + " !" + Chr( 13 )  , ;
         Chr( 13 ) + "Can`t find index field" + Chr( 13 ) + " !" + Chr( 13 )        , ;
         Chr( 13 ) + "  " + Chr( 13 ) + " memo  " + Chr( 13 )       , ;
         Chr( 13 ) + "   " + Chr( 13 )                                        , ;
         Chr( 13 ) + " " + Chr( 13 ) + "     " + Chr( 13 ) }

      _HMG_aABMLangLabel  := { ""    , ;
         "."        , ;
         "       ()"         , ;
         "     (.)"         , ;
         " ."  , ;
         ""               , ;
         " "      , ;
         " /"     , ;
         " "       , ;
         " "     , ;
         " Report"        , ;
         " "    , ;
         " "       , ;
         " "       , ;
         " of "          , ;
         ":"                , ;
         " :"      , ;
         " :"      , ;
         " by:"       , ;
         ""                  , ;
         ""                  , ;
         ". "                , ;
         " of "                 }

      _HMG_aABMLangButton := { ""    , ;
         "N"                 , ;
         ""            , ;
         ""            , ;
         ""              , ;
         ""             , ;
         ""             , ;
         ""               , ;
         "/"            , ;
         ""             , ;
         ""           , ;
         ""          , ;
         ""               , ;
         ""            , ;
         ""            , ;
         ""            , ;
         ""     }

      _HMG_aABMLangError  := { ",      "                 , ;
         ",       16 "                 , ;
         ",     (   bug)"    , ;
         ", main event number   (   bug)"   , ;
         ", list event number   (   bug)"  }

      // EDIT EXTENDED (GREEK - )

      _HMG_aLangButton := {   ;
      "&",       ; // 1
      "&",            ; // 2
      "&",       ; // 3
      "&",       ; // 4
      "&",         ; // 5
      "&",       ; // 6
      "&",          ; // 7
      "&Ok",             ; // 8
      "&",      ; // 9
      "&",         ; // 10
      "& ",   ; // 11
      "&",      ; // 12
		""        } // 13

      _HMG_aLangLabel := {            ;
      "",                       ; // 1
      "",                      ; // 2
      "",                       ; // 3
      " ",            ; // 4
      "",                     ; // 5
      " ",                  ; // 6
      ". ",              ; // 7
      " ",             ; // 8
      "",                       ; // 9
      " ",        ; // 10
      " ",              ; // 11
      "  ",           ; // 12
      " ",         ; // 13
      "  .",         ; // 14
      "  .",     ; // 15
      " ",            ; // 16
      "",                ; // 17
      " ",         ; // 18
      " : ",               ; // 19
      ": ",              ; // 20
      " " ,            ; // 21
      "  " ,       ; // 22
      " ",           ; // 23
      " ",                 ; // 24
      "   ",    ; // 25
      "  ",    ; // 26
      "",                         ; // 27
      " ",                     ; // 28
      " ",               ; // 29
      " ",                ; // 30
      "   ",         ; // 31
      "   "           } // 32

      _HMG_aLangUser := { ;
      CRLF + "    . "  + CRLF + ;
      "        EDIT   " + CRLF,                                          ; // 1
      "    ()",                                                                         ; // 2
      "    ()",                                                                         ; // 3
      " /",                                                                                       ; // 4
      "  ",                                                                                   ; // 5
      "   ",                                                                                   ; // 6
      "   &  OK",                                                                        ; // 7
      CRLF + "       " + CRLF + " ?    " + CRLF,              ; // 8
      CRLF + "    " + CRLF + "     " + CRLF,                ; // 9
      CRLF + "    memo  logic  " + CRLF,                                      ; // 10
      CRLF + "     " + CRLF,                                                           ; // 11
      "    ",                                                                        ; // 12
      "     ",                                                                       ; // 13
      " ",                                                                                       ; // 14
      "      ",                                                             ; // 15
      "      ",                                                                ; // 16
      "        ",                                                     ; // 17
      "Push button to select the last record to print",                                                          ; // 18
      CRLF + "    " + CRLF,                                                          ; // 19
      CRLF + "   " + CRLF,                                                             ; // 20
      CRLF + "    " + CRLF,                                                          ; // 21
      CRLF + "   " + CRLF,                                                             ; // 22
      CRLF + "    " + CRLF + "       " + CRLF, ; // 23
      CRLF + "   " + CRLF + "    " + CRLF,                       ; // 24
      CRLF + "     " + CRLF,                                                    ; // 25
      "  ",                                                                                         ; // 26
      " ",                                                                                             ; // 27
      " ",                                                                                             ; // 28
      "",                                                                                                     ; // 29
      "",                                                                                                     ; // 30
      ".:",                                                                                                   ; // 31
      CRLF + "   " + CRLF,                                                       ; // 32
      " by",                                                                                  ; // 33
      CRLF + "   " + CRLF,                                                            ; // 34
      CRLF + "     memo " + CRLF,                                            ; // 35
      CRLF + "     " + CRLF,                                                     ; // 36
      CRLF + "      " + CRLF,                                              ; // 37
      CRLF + "      " + CRLF,                                                     ; // 38
      CRLF + "    " + CRLF,                                                        ; // 39
      CRLF + " ;   " + CRLF,                                                             ; // 40
      CRLF + "        " + CRLF,                                            ; // 41
      CRLF + "You are going to restore the deleted record   " + CRLF + "Are you sure?    " + CRLF    } // 42

   CASE cLang == "BG"  // Bulgarian
      /////////////////////////////////////////////////////////////
      // BULGARIAN
      ////////////////////////////////////////////////////////////

      // MISC MESSAGES

      _HMG_MESSAGE [1] := '   ?'
      _HMG_MESSAGE [2] := '  '
      _HMG_MESSAGE [3] := '   '
      _HMG_MESSAGE [4] := '   '
      _HMG_MESSAGE [5] := ''
      _HMG_MESSAGE [6] := ''
      _HMG_MESSAGE [7] := ''
      _HMG_MESSAGE [8] := 'Apply'
      _HMG_MESSAGE [9] := '.'
      _HMG_MESSAGE [10] := 'Attention'
      _HMG_MESSAGE [11] := 'Information'
      _HMG_MESSAGE [12] := 'Stop'

      // BROWSE

      _HMG_BRWLangButton := { "" , ;
         ""    , ;
         ""      , ;
         "OK"           }
      _HMG_BRWLangError  := { ": "                                             , ;
         "   .  "                    , ;
         "MiniGUI "                                        , ;
         "  : "                                  , ;
         "  "                                                  , ;
         "   .  "                         , ;
         "Browse:     .  "    , ;
         "Browse: Append            .  ", ;
         "      "           , ;
         ""                                             , ;
         "   "                                 }
      _HMG_BRWLangMessage := { '   ?' , '  ' }

      // EDIT

      _HMG_aABMLangUser   := { Chr( 13 ) + "  ." + Chr( 13 ) + "   ?" + Chr( 13 )                  , ;
         Chr( 13 ) + "  " + Chr( 13 ) + "  " + Chr( 13 )   , ;
         Chr( 13 ) + "  " + Chr( 13 ) + "  " + Chr( 13 )   , ;
         Chr( 13 ) + "   " + Chr( 13 ) + "    " + Chr( 13 ) , ;
         Chr( 13 ) + "   " + Chr( 13 )                                       , ;
         Chr( 13 ) + "  " + Chr( 13 ) + "     " + Chr( 13 ) }
      _HMG_aABMLangLabel  := { ""              , ;
         " "       , ;
         "       ()"        , ;
         "   ()"        , ;
         " N:  ", ;
         ""               , ;
         " "        , ;
         " "         , ;
         " "        , ;
         "  " , ;
         "  "    , ;
         " "     , ;
         " "         , ;
         " "      , ;
         "   "          , ;
         ":"               , ;
         " :"        , ;
         " :"     , ;
         " :"       , ;
         ""                  , ;
         ""                  , ;
         " "           , ;
         "  "                 }
      _HMG_aABMLangButton := { ""   , ;
         ""       , ;
         ""   , ;
         ""   , ;
         ""     , ;
         " "   , ;
         ""     , ;
         ""    , ;
         ""     , ;
         ""    , ;
         "" , ;
         "" , ;
         ""    , ;
         ""  , ;
         ""   , ;
         ""    , ;
         ""    }
      _HMG_aABMLangError  := { "EDIT,      "                     , ;
         "EDIT,      16 "                      , ;
         "EDIT,       (  )", ;
         "EDIT,       (  )"   , ;
         "EDIT,         (  )" }

      // EDIT EXTENDED

      _HMG_aLangButton := {            ;
         "&",             ; // 1
      "&",               ; // 2
      "&",       ; // 3
      "&",            ; // 4
      "&",            ; // 5
      "&",             ; // 6
      "&",            ; // 7
      "&",                ; // 8
      "&",             ; // 9
      "&. ",       ; // 10
      "& ",      ; // 11
      "&",        ; // 12
		"Retry"               } // 13

      _HMG_aLangLabel := {            ;
         "",                         ; // 1
      "",                        ; // 2
      "",                       ; // 3
      "",                     ; // 4
      "",                    ; // 5
      " ",                    ; // 6
      " ",                ; // 7
      " ",                 ; // 8
      " ",                 ; // 9
      "  ",           ; // 10
      " ",              ; // 11
      "  ",              ; // 12
      " ",            ; // 13
      "   ",       ; // 14
      "   ",     ; // 15
      " ",                 ; // 16
      "",                      ; // 17
      "  ",         ; // 18
      "  : ",         ; // 19
      ": ",                     ; // 20
      "  " ,        ; // 21
      "  " ,            ; // 22
      "  ",       ; // 23
      "  ",          ; // 24
      "    ",     ; // 25
      "    ", ; // 26
      "",                        ; // 27
      "  ",                   ; // 28
      "-",                    ; // 29
      "-",                     ; // 30
      "-  ",          ; // 31
      "-  "         }    // 32
      _HMG_aLangUser := { ;
         CRLF + "    ."  + CRLF + "     EDIT" + CRLF, ; // 1
      " ",                                                                                     ; // 2
      " ",                                                                                     ; // 3
      " ",                                                                                      ; // 4
      " ",                                                                                 ; // 5
      "   ",                                                                       ; // 6
      "    OK",                                                                      ; // 7
      CRLF + "     " + CRLF + "   ?    " + CRLF,                       ; // 8
      CRLF + "  " + CRLF + "   " + CRLF,                            ; // 9
      CRLF + "  MEMO       " + CRLF,                         ; // 10
      CRLF + "     " + CRLF,                                                                ; // 11
      "      ",                                                             ; // 12
      "    ",                                                                           ; // 13
      "  ",                                                                                   ; // 14
      "        ",                                                  ; // 15
      "        ",                                              ; // 16
      ",    ",                                                                     ; // 17
      ",    ",                                                                    ; // 18
      CRLF + "   " + CRLF,                                                                ; // 19
      CRLF + "    " + CRLF,                                                             ; // 20
      CRLF + "    " + CRLF,                                                           ; // 21
      CRLF + "    " + CRLF,                                                            ; // 22
      CRLF + "   " + CRLF + "    " + CRLF,                          ; // 23
      CRLF + "   " + CRLF + "   " + CRLF,                      ; // 24
      CRLF + "     " + CRLF,                                                                ; // 25
      " ",                                                                                          ; // 26
      "  ",                                                                                          ; // 27
      "  ",                                                                                          ; // 28
      "",                                                                                                 ; // 29
      "",                                                                                                 ; // 30
      ":",                                                                                          ; // 31
      CRLF + "   " + CRLF,                                                                   ; // 32
      " ",                                                                                      ; // 33
      CRLF + "        " + CRLF,                                                        ; // 34
      CRLF + "MEMO      " + CRLF,                                                      ; // 35
      CRLF + "       " + CRLF,                                                       ; // 36
      CRLF + "   " + CRLF,                                                         ; // 37
      CRLF + "   " + CRLF,                                                         ; // 38
      CRLF + "     " + CRLF,                                                               ; // 39
      CRLF + "  ?   " + CRLF,                                                                   ; // 40
      CRLF + "      " + CRLF,                                                ; // 41
      CRLF + "     " + CRLF + "   ?    " + CRLF                    } // 42

   ENDCASE

#endif

RETURN
